<?php

namespace Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\Helpers;

class ParentRelatedListRetriever
{
    protected $relateFields;

    protected $blackListedRelateFields = array();

    public function __construct()
    {
    }

    public function setRelateParentList(\SugarBean $eventBean, \SugarBean $targetEntityBean, array $linkNames)
    {
        if (empty($eventBean) || empty($targetEntityBean) || empty($linkNames)) {
            return array();
        }

        $relatedData = $this->getRelatedData($targetEntityBean, $linkNames);
        $this->createParentRelatedConnections($eventBean, $relatedData);

        //return $this->relateFields[$targetModule];
    }

    protected function getRelatedData($targetEntityBean, $linkNames)
    {
        $relatedData = array();

        foreach($linkNames as $linkData){
            $firstLevelLinkName = $linkData['linkLevel1'];
            if(empty($firstLevelLinkName))
                continue;

            if(!$targetEntityBean->load_relationship($firstLevelLinkName))
                continue;

            $link = $targetEntityBean->getFieldValue($firstLevelLinkName);
            if(!empty($link)){
                $relatedIds = array();
                $relatedModuleName = $link->getRelatedModuleName();

                if($link->isParentRelationship()){
                    if(!empty($targetEntityBean->parent_type) && !empty($targetEntityBean->parent_id) && $targetEntityBean->parent_type == $link->getRelatedModuleName()){
                        $relatedIds = array($targetEntityBean->parent_id);
                    }
                }else {
                    $relatedIds = $link->get();
                }

                if(!empty($relatedIds)){
                    $relatedData[] = array(
                        'relatedModule' => $relatedModuleName,
                        'relatedIds' => $relatedIds,
                        'linkName' => $firstLevelLinkName,
                    );
                }
            }
        }

        return $relatedData;
    }

    protected function createParentRelatedConnections(\SugarBean $eventBean, array $relatedData){
//        $this->deleteOldLinks($eventBean->id);
//        global $timedate, $current_user;
//        $db = \DBManagerFactory::getInstance();
//        $currentDatetime = $timedate->nowDb();
//        $currentUserId = $current_user->id;
        foreach($relatedData as $relatedItem) {
            $parentType = $relatedItem['relatedModule'];
            $linkName = $relatedItem['linkName'];
            foreach($relatedItem['relatedIds'] as $parentId) {
                $ntdvRelates = \BeanFactory::getBean('PX_NeedToDoRelates');
                $ntdvRelates->parent_type = $parentType;
                $ntdvRelates->parent_id = $parentId;
                $ntdvRelates->relate_type = 'Target';
                $ntdvRelates->event_id = $eventBean->id;
                $ntdvRelates->link_name = $linkName;
                $ntdvRelates->save();

//                $ntdvParentsId = \Sugarcrm\Sugarcrm\Util\Uuid::uuid1();
//                $insertSQL = "
//                    INSERT INTO px_ntdv_parents
//                      (id, date_entered, created_by, description, parent_type, parent_id, event_id, link_name)
//                    VALUES
//                      (
//                        '$ntdvParentsId',
//                        '$currentDatetime',
//                        '$currentUserId',
//                        '',
//                        '$parentType',
//                        '$parentId',
//                        '{$eventBean->id}',
//                        '$linkName'
//                      )
//                ";
//                $db->query($insertSQL);
            }
        }
    }

    public function deleteOldLinks($eventId){
        $db = \DBManagerFactory::getInstance();
        $cleanUpSQL = "DELETE FROM px_ntdv_parents WHERE event_id='$eventId'";
        $db->query($cleanUpSQL);
    }


    protected function fieldHasNameAttribute($fieldDefs){
        return !empty($fieldDefs['name']);
    }

    protected function fieldIsRelated($fieldDefs){
        return !empty($fieldDefs['type']) && $fieldDefs['type'] == 'link';
    }

    protected function relateFieldIsValid($targetBean, $fieldDefs){
        $relationship = $targetBean->load_relationship($fieldDefs['name']);
        return !empty($relationship);
    }

    protected function getFieldLabel($targetBean, $fieldDefs){
        $targetModule = $targetBean->getModuleName();
        $fieldName = $fieldDefs['name'];
        $labelKey = !empty($fieldDefs['vname']) ? $fieldDefs['vname'] : '';
        $labelValue = translate($labelKey, $targetModule);
        $relatedModuleLabel = $this->getRelatedModuleLabel($targetBean, $fieldName);

        if(empty($labelValue)){
            $labelValue = $fieldName;
        }
        $labelValue = $this->removeColonFromTheEndOfSomeLabels($labelValue);
        return $this->translateFieldLabel($relatedModuleLabel, $fieldName, $labelValue);
    }

    protected function translateFieldLabel($moduleLabel, $linkName, $linkLabel){
        $moduleLabelLower = mb_strtolower($moduleLabel);
        $linkNameLower = mb_strtolower($linkName);
        $linkLabelLower = mb_strtolower($linkLabel);

        $translateString = 'LBL_ENTITY_PARENT_ITEM_TPL_MODULE';
        if($linkLabelLower != $moduleLabelLower && $linkLabelLower != $linkNameLower){
            $translateString = 'LBL_ENTITY_PARENT_ITEM_TPL_LINK_AND_MODULE';
        }else if($linkLabelLower != $moduleLabelLower){
            $translateString = 'LBL_ENTITY_PARENT_ITEM_TPL_MODULE';
        }else if($linkLabelLower != $linkNameLower){
            $translateString = 'LBL_ENTITY_PARENT_ITEM_TPL_LINK';
        }

        $tpl = translate($translateString, 'PX_NeedToDoEventTypes');
        return str_replace(array('%LINK_LABEL%', '%LINK_NAME%', '%RELATED_MODULE_NAME%'), array($linkLabel, $linkName, $moduleLabel), $tpl);
    }

    protected function getRelatedModuleLabel($targetBean, $linkName){
        $link = $targetBean->getFieldValue($linkName);

        if(empty($link) || !method_exists($link, 'getRelatedModuleName') || !method_exists($link, 'getType'))
            return '';

        $linkType = $link->getType();
        $relatedModuleName = $link->getRelatedModuleName();

        if($linkType == 'one'){
            $relatedModuleLabel = $GLOBALS['app_list_strings']['moduleListSingular'][$relatedModuleName];
        }elseif($linkType == 'many'){
            $relatedModuleLabel = $GLOBALS['app_list_strings']['moduleList'][$relatedModuleName];
        }

        if(empty($relatedModuleLabel))
            $relatedModuleLabel = $relatedModuleName;

        return $relatedModuleLabel;
    }

    private function removeColonFromTheEndOfSomeLabels($label){
        return preg_replace ( '/(.*)(:+)$/' , '$1' , trim($label));

    }

}