<?php

namespace Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\Helpers;

class TargetEntityParentList
{
    protected $relateFields;

    protected $blackListedRelateFields = array();

    public function __construct()
    {
    }

    public function getRelateFields($targetModule = '')
    {
        if (empty($targetModule)) {
            return array();
        }elseif (!isset($this->relateFields[$targetModule])) {
            $this->setRelatedFields($targetModule);
        }
        return $this->relateFields[$targetModule];
    }

    protected function setRelatedFields($targetModule)
    {
        $targetBean = \BeanFactory::getBean($targetModule);
        if(empty($targetBean)){
            $this->relateFields[$targetModule] = array();
            return;
        }

        $relatedFields = array();

        foreach ($targetBean->field_defs as $fieldDefs) {
            if (
                $this->fieldIsRelated($fieldDefs) &&
                $this->fieldHasNameAttribute($fieldDefs) &&
                $this->relateFieldIsValid($targetBean, $fieldDefs)
            ) {
                $relatedFields[$fieldDefs['name']] = $this->getFieldLabel($targetBean, $fieldDefs);
            }
        }

        $this->relateFields[$targetModule] = $relatedFields;
//        return $this->relateFields[$targetModule];
    }

    protected function fieldHasNameAttribute($fieldDefs){
        return !empty($fieldDefs['name']);
    }

    protected function fieldIsRelated($fieldDefs){
        return !empty($fieldDefs['type']) && $fieldDefs['type'] == 'link';
    }

    protected function relateFieldIsValid($targetBean, $fieldDefs){
        $relationship = $targetBean->load_relationship($fieldDefs['name']);
        return !empty($relationship);
    }

    protected function getFieldLabel($targetBean, $fieldDefs){
        $targetModule = $targetBean->getModuleName();
        $fieldName = $fieldDefs['name'];
        $labelKey = !empty($fieldDefs['vname']) ? $fieldDefs['vname'] : '';
        $labelValue = translate($labelKey, $targetModule);
        $relatedModuleLabel = $this->getRelatedModuleLabel($targetBean, $fieldName);

        if(empty($labelValue)){
            $labelValue = $fieldName;
        }
        $labelValue = $this->removeColonFromTheEndOfSomeLabels($labelValue);
        return $this->translateFieldLabel($relatedModuleLabel, $fieldName, $labelValue);
    }

    protected function translateFieldLabel($moduleLabel, $linkName, $linkLabel){
        $moduleLabelLower = mb_strtolower($moduleLabel);
        $linkNameLower = mb_strtolower($linkName);
        $linkLabelLower = mb_strtolower($linkLabel);

        $translateString = 'LBL_ENTITY_PARENT_ITEM_TPL_MODULE';
        if($linkLabelLower != $moduleLabelLower && $linkLabelLower != $linkNameLower){
            $translateString = 'LBL_ENTITY_PARENT_ITEM_TPL_LINK_AND_MODULE';
        }else if($linkLabelLower != $moduleLabelLower){
            $translateString = 'LBL_ENTITY_PARENT_ITEM_TPL_MODULE';
        }else if($linkLabelLower != $linkNameLower){
            $translateString = 'LBL_ENTITY_PARENT_ITEM_TPL_LINK';
        }

        $tpl = translate($translateString, 'PX_NeedToDoEventTypes');
        return str_replace(array('%LINK_LABEL%', '%LINK_NAME%', '%RELATED_MODULE_NAME%'), array($linkLabel, $linkName, $moduleLabel), $tpl);
    }

    protected function getRelatedModuleLabel($targetBean, $linkName){
        $link = $targetBean->getFieldValue($linkName);

        if(empty($link) || !method_exists($link, 'getRelatedModuleName') || !method_exists($link, 'getType'))
            return '';

        $linkType = $link->getType();
        $relatedModuleName = $link->getRelatedModuleName();

        if($linkType == 'one'){
            $relatedModuleLabel = $GLOBALS['app_list_strings']['moduleListSingular'][$relatedModuleName];
        }elseif($linkType == 'many'){
            $relatedModuleLabel = $GLOBALS['app_list_strings']['moduleList'][$relatedModuleName];
        }

        if(empty($relatedModuleLabel))
            $relatedModuleLabel = $relatedModuleName;

        return $relatedModuleLabel;
    }

    private function removeColonFromTheEndOfSomeLabels($label){
        return preg_replace ( '/(.*)(:+)$/' , '$1' , trim($label));

    }

}