<?php

class PxNtdvGlobalLogicHooks
{
    const INDEX_STATUS_IN_PROCESS = 'In Process';
    protected static $targetEventTypes;
    protected static $relatedData;

    public function needToDoEventModulesIsInstalled(){
        $needToDoEventTypeBean = \BeanFactory::getBean('PX_NeedToDoEventTypes');
        return !empty($needToDoEventTypeBean);
    }

    public function firstBeforeSave(\SugarBean $bean, $event, $args)
    {
        if(!$this->needToDoEventModulesIsInstalled()) {
            return;
        }

        $this->collectEventTypes();
    }

    public function firstBeforeRelationshipAdd($bean, $event, $arguments){
        if(!$this->needToDoEventModulesIsInstalled()) {
            return;
        }

        $this->collectEventTypes();
    }

    public function firstBeforeRelationshipDelete($bean, $event, $arguments){
        if(!$this->needToDoEventModulesIsInstalled()) {
            return;
        }

        $this->collectEventTypes();
    }

    protected function collectEventTypes(){
        if(isset(self::$targetEventTypes) || !$this->needToDoEventModulesIsInstalled()) {
            return;
        }

        $needToDoEventTypes = \BeanFactory::getBean('PX_NeedToDoEventTypes');

        $sugarQuery = new \SugarQuery();
        $sugarQuery->from($needToDoEventTypes);
        $sugarQuery->select(array('id', 'target_module', 'target_attribute_name', 'target_entity_filter', 'target_entity_parent_list'));
        $sugarQuery->where()->equals('status', \PX_NeedToDoEventTypes::ENABLED_STATUS);
        $queryResult = $sugarQuery->execute();
        foreach($queryResult as $row){
            $row['target_entity_parent_list'] = $needToDoEventTypes->getParentRelatedNames($row['target_entity_parent_list']);
            if(!empty($row['target_entity_parent_list'])) {
                $targetModule = $row['target_module'];
                $relatedBean = \BeanFactory::getBean($targetModule);
                foreach($row['target_entity_parent_list'] as $parentItem) {
                    $linkName = $parentItem['linkLevel1'];
                    if(!$relatedBean->load_relationship($linkName))
                        continue;

                    $link = $relatedBean->getFieldValue($linkName);
                    $relationshipName = $relatedBean->field_defs[$linkName]['relationship'];

                    self::$relatedData[] = array(
                        'targetModule' => $targetModule,
                        'targetLink' => $linkName,
                        'relatedModule' => $link->getRelatedModuleName(),
                        'relationship' => $relationshipName,
                        'eventTypeId' => $row['id'],
                    );
                }
            }
            self::$targetEventTypes[$row['target_module']][] = $row;
        }
    }

    public function lastAfterSave(\SugarBean $bean, $event, $args)
    {
        if(!$this->needToDoEventModulesIsInstalled() || empty(self::$targetEventTypes)) {
            return;
        }

        $targetModule = $bean->getModuleName();
        if(empty(self::$targetEventTypes[$targetModule])) {
            return;
        }

        $this->pushRecordToIndexing($bean, $args, self::$targetEventTypes[$targetModule]);
    }

    protected function pushRecordToIndexing(\SugarBean $bean, $args, $needToDoEventTypes){
        global $timedate, $current_user;
        $db = \DBManagerFactory::getInstance();
        $currentDatetime = $timedate->nowDb();
        $currentUserId = $current_user->id;
        $needToDoEventTypeBean = \BeanFactory::getBean('PX_NeedToDoEventTypes');
        $needToDoEventBean = \BeanFactory::getBean('PX_NeedToDoEvents');

        foreach($needToDoEventTypes as $needToDoEventTypeData) {
            $indexStackId = \Sugarcrm\Sugarcrm\Util\Uuid::uuid1();
            $targetModule = $bean->getModuleName();
            $targetRecordId = $bean->id;
            $targetAttrName = $needToDoEventTypeData['target_attribute_name'];
            $targetControlDate = $bean->getFieldValue($targetAttrName); //$args['dataChanges'][$targetAttrName]['after'];
            $needToDoEventTypeId = $needToDoEventTypeData['id'];

//            $GLOBALS['log']->fatal('$targetControlDate: ' . print_r($targetControlDate, true));
            $targetControlDateType = $needToDoEventTypeBean->getTargetAttributeType($targetModule, $targetAttrName);
            $targetControlDate = $needToDoEventBean->getControlDateForDateFormat($targetControlDate, $targetControlDateType);

//            if(empty($targetControlDate))
//                continue;

            $stackIdSQL = "
                SELECT id 
                FROM px_ntdv_indexing_stack 
                WHERE 
                    target_module='$targetModule' AND 
                    target_record_id='$targetRecordId' AND 
                    need_to_do_event_type_id='$needToDoEventTypeId' AND
                    (status IS NULL OR status = '' OR status <> '" . self::INDEX_STATUS_IN_PROCESS . "')
            ";
//            $rs = $db->query($stackIdSQL);
            $row = $db->fetchOne($stackIdSQL);
            if(!empty($row['id'])){
                $updateSQL = "
                    UPDATE px_ntdv_indexing_stack
                    SET 
                        target_control_date='$targetControlDate',
                        date_entered='$currentDatetime',
                        created_by='$currentUserId'
                    WHERE 
                        id='" . $row['id'] . "'        
                ";
                $db->query($updateSQL);
            }else {
                $insertSQL = "
                    INSERT INTO px_ntdv_indexing_stack 
                      (id, target_module, target_record_id, need_to_do_event_type_id, target_control_date, date_entered, created_by, description)
                    VALUES 
                      (
                        '" . $indexStackId . "',
                        '" . $targetModule . "',
                        '" . $targetRecordId . "',
                        '" . $needToDoEventTypeId . "',
                        " . (empty($targetControlDate) ? 'NULL' : ("'" . $targetControlDate . "'")) . ",
                        '" . $currentDatetime . "',
                        '" . $currentUserId . "',
                        ''
                      )
                ";
                $db->query($insertSQL);
            }
        }
    }

    public function firstAfterRelationshipAdd($bean, $event, $arguments){
        if(!$this->needToDoEventModulesIsInstalled()) {
            return;
        }
        $indexRelations = new \Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\Indexing\IndexRelations();

        foreach(self::$relatedData as $item) {
            if($arguments['relationship'] != $item['relationship']) {
                continue;
            }

            if (
                (
                    $arguments['module'] == $item['relatedModule'] &&
                    $arguments['related_module'] == $item['targetModule']
                ) || (
                    $arguments['related_module'] == $item['relatedModule'] &&
                    $arguments['module'] == $item['targetModule']
                )
            ){

                if($arguments['module'] == $item['relatedModule']){
                    $relatedId = $arguments['id'];
                    $recordId = $arguments['related_id'];
                }elseif($arguments['related_module'] == $item['relatedModule']){
                    $relatedId = $arguments['related_id'];
                    $recordId = $arguments['id'];
                }

                $indexRelations->addRelation(array(
                        'module_name' => $item['targetModule'],
                        'record_id' => $recordId,
                        'event_type_id' => $item['eventTypeId'],
                        'related_module_name' => $item['relatedModule'],
                        'related_record_id' => $relatedId,
                        'link_name' => $item['targetLink'],
                    )
                );

                    //$item['targetModule'], $recordId, $item['eventTypeId'], $item['relatedModule'], $relatedId);
            }
        }
    }

    public function firstAfterRelationshipDelete($bean, $event, $arguments){
        if(!$this->needToDoEventModulesIsInstalled()) {
            return;
        }

        $indexRelations = new \Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\Indexing\IndexRelations();

        foreach(self::$relatedData as $item) {
            if($arguments['relationship'] != $item['relationship'])
                continue;

            if (
                (
                    $arguments['module'] == $item['relatedModule'] &&
                    $arguments['related_module'] == $item['targetModule']
                ) || (
                    $arguments['related_module'] == $item['relatedModule'] &&
                    $arguments['module'] == $item['targetModule']
                )
            ){
                if($arguments['module'] == $item['relatedModule']){
                    $relatedId = $arguments['id'];
                    $recordId = $arguments['related_id'];
                }elseif($arguments['related_module'] == $item['relatedModule']){
                    $relatedId = $arguments['related_id'];
                    $recordId = $arguments['id'];
                }

                $indexRelations->removeRelation(array(
                        'module_name' => $item['targetModule'],
                        'record_id' => $recordId,
                        'event_type_id' => $item['eventTypeId'],
                        'related_module_name' => $item['relatedModule'],
                        'related_record_id' => $relatedId,
                        'link_name' => $item['targetLink'],
                    )
                );
            }
        }
    }

}