<?php

namespace Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\PX_NeedToDoViewerComments\Helpers\Notification\MessageComposer;

use \Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\PX_NeedToDoViewerComments\Helpers\SugarUrlGenerator;

class MessageComposer{

    protected $messageTemplate;
    protected $event;
    protected $urlGenerator = null;



    public function __construct(SugarUrlGenerator $urlGenerator){
        $this->messageTemplate = new MessageTemplate();
        $this->urlGenerator = $urlGenerator;
    }

    public function getTitle($user, $event){

        $template = $this->messageTemplate->getTitleTemplate($user, $event->getLangKey());

        $data = $event->getData();
        $createdBy = $this->getEntityName($data['CREATED_BY']);

        $title = str_replace(['{{CREATED_BY}}'], [$createdBy], $template);
        return $title;
    }

    public function getText($user, $event){
        $template = $this->messageTemplate->getBodyTemplate($user, $event->getLangKey());

        $data = $event->getData();
        $createdBy = $this->getEntityName($data['CREATED_BY']);
        $commentText = $data['COMMENT_TEXT'];
        $dateEntered = $this->getDateFormatted($data['DATE_ENTERED']);
        $parentLink = $this->getEntityName($data['NEED_TO_DO_EVENT_BEAN']);

        $text = str_replace(
            ['{{COMMENT_TEXT}}', '{{DATE_ENTERED}}', '{{CREATED_BY}}', '{{PARENT_LINK}}'],
            [$commentText, $dateEntered, $createdBy, $parentLink],
            $template);
        return $text;
    }

    protected function getEntityName($bean, $user = null){
        if(empty($bean))
            return '';
        else if($bean instanceof \Person){
            return $bean->full_name;
        }else{
            return $bean->name;
        }
    }

    protected function getDateFormatted($date, $user = null){
        if(empty($date))
            return '';
        global $timedate;
        return $timedate->to_display_date_time($date, true, true, $user);
    }

    protected function getHtmlLinkToRecord($entity){
        if(empty($entity) || empty($entity->id))
            return '';

        $url = $this->urlGenerator->getSugarRecordUrl($entity->module_name, $entity->id);
        $name = $this->getEntityName($entity);
        return '<a href="' . $url . '" target="_blank">' . $name . '</a>';
    }

    public function getBodyTemplate(\User $user, $templateKey){
        $bodyTemplateKey = self::LABEL_PREFIX . $templateKey . self::BODY_SUFFIX;
        $tpls = $this->getTemplateList($user);

        if(!empty($tpls[$bodyTemplateKey]))
            return $tpls[$bodyTemplateKey];

        return '';
    }

    public function getTemplateList(\User $user){
        if(!isset($this->templates[$user->preferred_language])){
            $this->templates[$user->preferred_language] = $this->loadTemplates($user->preferred_language);
        }
        return $this->templates[$user->preferred_language];
    }

    protected function loadTemplates($preferencedLanguage){
        $languageList = $this->getLanguageList($preferencedLanguage);
        $templateList = array();

        foreach ($languageList as $language) {
            foreach(SugarAutoLoader::existing(
                "custom/PxRoutines/NeedToDoViewer/PX_NeedToDoViewerComments/Helpers/Notification/MessageComposer/language/$language.templates.php"
            ) as $file) {
                include $file;
            }
            $templateList = sugarLangArrayMerge($templateList, $templates);
        }

        return $templateList;
    }

    protected function getLanguageList($preferencedLanguage){
        global $sugar_config;

        $defaultLanguage = !empty($sugar_config['default_language']) ? $sugar_config['default_language'] : $preferencedLanguage;
        $languageList = array();
        if ($preferencedLanguage != 'en_us'){
            $languageList[] = 'en_us';
        }
        if ($defaultLanguage != 'en_us' && $preferencedLanguage != $defaultLanguage){
            $languageList[] = $defaultLanguage;
        }
        $languageList[] = $preferencedLanguage;

        return $languageList;
    }

}