<?php

namespace Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\Package\License;

use Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\Package\License\Validations\Validation;
use Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\Package\License\Exceptions\InvalidLicenseException;
use Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\Package\Package;

require_once 'modules/PX_NeedToDoEvents/PX_NeedToDoEvents.php';

class Control
{

    /**
     * @var Config
     */
    protected $config;

    protected $validation;

    /**
     * @var Decryptor
     */
    protected $decryptor;

    protected static $checkedLicenses = array();

    const GENERAL_LICENSE_STATE_KEY = 'GENERAL_LICENSE_STATE_KEY';

    public function __construct(){
        $this->setConfig();
    }

    public function checkLicense()
    {
        if($this->isLicenseChecked(self::GENERAL_LICENSE_STATE_KEY)){
            return $this->getLicenseState(self::GENERAL_LICENSE_STATE_KEY);
        }

        $package = new Package();
        $licenseType = $package->getPackageLicenseType();
        if($licenseType == Package::SUGAR_OUTFITTERS_LICENSE_TYPE) {
            $this->checkSugarOutfittersLicense();
        }else if($licenseType == Package::NATIVE_LICENSE_TYPE){
            $this->checkNativeLicense();
        }

        return $this->getLicenseState(self::GENERAL_LICENSE_STATE_KEY);
    }

    private function checkNativeLicense(){
        try {
            $this->getValidation()->validateLicense();
            $this->setLicenseState(self::GENERAL_LICENSE_STATE_KEY, true);
        }catch(\Exception $e){
            $this->setLicenseState(self::GENERAL_LICENSE_STATE_KEY, false);
            throw $e;
        }
    }

    private function checkSugarOutfittersLicense(){
        require_once('modules/PX_NeedToDoOutfittersLicense/license/PX_NeedToDoOutfittersLicenseHandler.php');
        try {
            $licenseState = \PX_NeedToDoOutfittersLicenseHandler::isValid('PX_NeedToDoOutfittersLicense');
            if($licenseState !== true && $licenseState !== false)
                throw new InvalidLicenseException($licenseState);
            $this->setLicenseState(self::GENERAL_LICENSE_STATE_KEY, $licenseState);
        }catch(\Exception $e){
            $this->setLicenseState(self::GENERAL_LICENSE_STATE_KEY, false);
            throw $e;
        }
    }

    public function checkLicenseForCurrentUser()
    {
        $package = new Package();
        $licenseType = $package->getPackageLicenseType();
        if($licenseType == Package::SUGAR_OUTFITTERS_LICENSE_TYPE) {
            $this->checkSugarOutfittersLicenseForCurrentUser();
        }else if($licenseType == Package::NATIVE_LICENSE_TYPE){
            $this->checkNativeLicenseForCurrentUser();
        }else {
            throw new InvalidLicenseException(translate('ERROR_NTDV_LICENSE_TYPE_NOT_FOUND', 'PX_NeedToDoEvents'));
        }
    }

    private function checkNativeLicenseForCurrentUser(){
        try{
            if(!$this->checkLicense())
                throw new \Exception('');
        }catch(\Exception $e){
            throw new InvalidLicenseException(translate('ERROR_NTDV_INVALID_LICENSE_GENERAL', 'PX_NeedToDoEvents'));
        }

        global $current_user;
        if($this->isLicenseChecked($current_user->id)){
            if($this->getLicenseState($current_user->id)){
                return;
            }
            throw new InvalidLicenseException(translate('ERROR_NTDV_INVALID_LICENSE_CURRENT_USER', 'PX_NeedToDoEvents'));
        }

        try {
            $this->getValidation()->validateUser($current_user);
            $this->setLicenseState($current_user->id, true);
        }catch(\Exception $e){
            throw $e;
        }
    }

    private function checkSugarOutfittersLicenseForCurrentUser(){
        global $current_user;

        if($this->isLicenseChecked($current_user->id)){
            if($this->getLicenseState($current_user->id)){
                return;
            }
            throw new InvalidLicenseException(translate('ERROR_NTDV_INVALID_LICENSE_CURRENT_USER', 'PX_NeedToDoEvents'));
        }

        require_once('modules/PX_NeedToDoOutfittersLicense/license/PX_NeedToDoOutfittersLicenseHandler.php');

        try {
            $licenseState = \PX_NeedToDoOutfittersLicenseHandler::isValid('PX_NeedToDoOutfittersLicense', $current_user->id);
            if($licenseState !== true && $licenseState !== false)
                throw new InvalidLicenseException($licenseState);
            $this->setLicenseState($current_user->id, $licenseState);
        }catch(\Exception $e){
            $this->setLicenseState($current_user->id, false);
            throw $e;
        }
    }

    protected function isLicenseChecked($key){
        return isset(static::$checkedLicenses[$key]);
    }

    protected function setLicenseState($key, $state){
        static::$checkedLicenses[$key] = $state;
    }

    protected function getLicenseState($key){
        return isset(static::$checkedLicenses[$key]) ? static::$checkedLicenses[$key] : false;
    }

    /**
     * @return Validation
     */
    protected function getValidation()
    {
        if(empty($this->validation)){
            $this->setValidation();
        }
        return $this->validation;
    }

    protected function setValidation()
    {
        $validationFactory = new ValidationFactory(
            $this->getDecryptedToken(),
            $this->getConfig()->package_installation_date
        );
        $this->validation = $validationFactory->getValidation();
    }

    /**
     * @return Decryptor
     */
    protected function getDecryptor(){
        if(!isset($this->decryptor)){
            $this->setDectyptor();
        }
        return $this->decryptor;
    }

    protected function setDectyptor(){
        $this->decryptor = new Decryptor('custom/PxRoutines/NeedToDoViewer/Package/License/Key/public_key.php');
    }

    protected function getDecryptedToken(){
        return $this->getDecryptor()->decrypt($this->getLicenseToken());
    }

    /**
     * @return Config
     */
    public function getConfig()
    {
        if(empty($this->config)){
            $this->setConfig();
        }
        return $this->config;
    }

    /**
     *
     */
    protected function setConfig()
    {
        $this->config = new Config();
    }

    public function setLicenseToken($licenseToken){
        $this->getConfig()->license_token = $licenseToken;
    }

    public function getLicenseToken()
    {
        return $this->getConfig()->license_token;
    }

    public function getUserLimit()
    {
        return $this->getValidation()->getUsersLimit();
    }

    public function getCurrentUsers()
    {
        return $this->getValidation()->getCurrentUsers();
    }

    public function getExpireDate()
    {
        return $this->getValidation()->getExpireDate();
    }

    public function getLicenseType()
    {
        return $this->getValidation()->getLicenseType();
    }

    public static function clearCachedData(){
        $validationFactory = new ValidationFactory('',null);
        $validation = $validationFactory->getValidation();
        $validation->clearCachedData();
    }
}
