<?php

if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');

require_once 'clients/base/api/FilterApi.php';

class PX_NeedToDoCollaborationApi extends FilterApi{

    public function registerApiRest()
    {
        return array(
            'NtdvCollaborationDialogList' => array(
                'reqType' => 'GET',
                'path' => array('px-need-to-do-collaboration', 'dialog-list'),
                'pathVars' => array('', ''),
                'method' => 'getCollaborationDialogList',
                'shortHelp' => '',
                'longHelp' => '',
            ),
            'NtdvCollaborationDialogHistory' => array(
                'reqType' => 'GET',
                'path' => array('px-need-to-do-collaboration', 'dialog-list', '?'),
                'pathVars' => array('', '', 'dialogId'),
                'method' => 'CollaborationDialogHistory',
                'shortHelp' => '',
                'longHelp' => '',
            ),
            'NtdvCollaborationReadMessage' => array(
                'reqType' => 'PUT',
                'path' => array('px-need-to-do-collaboration', 'read-message', '?'),
                'pathVars' => array('', '', 'messageId'),
                'method' => 'CollaborationReadMessage',
                'shortHelp' => '',
                'longHelp' => '',
            ),
            'NtdvCollaborationMessageStatistics' => array(
                'reqType' => 'GET',
                'path' => array('px-need-to-do-collaboration', 'statistics', '?'),
                'pathVars' => array('', '', 'messageId'),
                'method' => 'CollaborationMessageStatistics',
                'shortHelp' => '',
                'longHelp' => '',
            )
        );
    }

    public function getCollaborationDialogList(ServiceBase $api, array $args){
        $args['module'] = 'PX_NeedToDoViewerComments';
        $params = $this->parseArguments($api, $args);
        $query = $this->getDialogListQueryObject($api, $params);
        return $this->formatResult($api, $args, $query);
    }

//    protected function parseArguments(ServiceBase $api, array $args)
    protected function parseArguments(ServiceBase $api, array $args, SugarBean $seed = null)
    {
        $params = parent::parseArguments($api, $args);
        if (isset($args['filter'])) {
            $params['filter'] = $args['filter'];
        }
        return $params;
    }

    protected function getDialogListQueryObject(ServiceBase $api, array $params){
        $seed = BeanFactory::getBean('PX_NeedToDoViewerComments');
        $query = new SugarQuery();
        $query->from($seed);

        $needToDoJoin = $query->join('need_to_do_event', array());
        $needToDoJoinA = $needToDoJoin->joinName();

        global $current_user;
        $subscribedJTA = 'subscribed_users';
        $subscribedTableSQL = "(SELECT id, created_by, parent_type, parent_id from subscriptions WHERE deleted=0)";
        $query->join[$subscribedJTA] = $query->joinTable($subscribedTableSQL, array('alias' => $subscribedJTA, 'joinType' => 'LEFT'));
        $query->join[$subscribedJTA]->on()->equalsField($subscribedJTA . '.parent_id', 'px_needtodoviewercomments.id')
            ->equals($subscribedJTA . '.parent_type', 'PX_NeedToDoViewerComments')
            ->equals($subscribedJTA . '.created_by', $current_user->id);

        $query->where()->equals('parent_type', 'PX_NeedToDoEvents');

        if (!empty($params['filter'])) {
            $this->applyParticipantsFilter($params['filter'], $query);
            $this->applyMentionedFilter($params['filter'], $query);
            $this->applyNewMessagesFilter($params['filter'], $query);
            $this->applyMyNewMessagesFilter($params['filter'], $query);
            $this->applyMySubscriptionsFilter($params['filter'], $query);
            $this->applySubscriptionFilter($params['filter'], $query);
            $this->applyRelevantToFilter($params['filter'], $query);
            $this->applyRelevantToMeFilter($params['filter'], $query);
            $this->applyNeedToDoEventsFilter($params['filter'], $query, $needToDoJoinA);
            self::addFilters($params['filter'], $query->where(), $query);
        }

        $query->select(array('id', 'name', 'date_entered', 'date_modified'));
        $query->select()->addField($needToDoJoinA.'.id', array('alias' => 'needtodo_id'));
        $query->select()->addField($needToDoJoinA.'.name', array('alias' => 'needtodo_name'));
        $query->select()->addField($needToDoJoinA.'.name_html', array('alias' => 'needtodo_name_html'));
        $query->select()->addField($needToDoJoinA.'.event_type_id', array('alias' => 'needtodo_event_type_id'));
        $query->select()->addField($needToDoJoinA.'.parent_type', array('alias' => 'needtodo_target_type'));
        $query->select()->addField($needToDoJoinA.'.parent_id', array('alias' => 'needtodo_target_id'));
        $query->select()->addField($subscribedJTA . '.id', array('alias' => 'subscription_id'));

        $query->limit($params['limit'] + 1)->offset($params['offset']);
        $query->orderBy('date_modified', 'DESC');

//        $GLOBALS['log']->fatal('QUERY: ' . print_r($query->compile()->getSQL(), true));
//        $GLOBALS['log']->fatal('QUERY: ' . print_r($query->compile()->getParameters(), true));

        return $query;
    }

    protected function applyParticipantsFilter(&$filterList, SugarQuery $query){
        foreach($filterList as $key => $filter){
            foreach($filter as $fieldName => $filterDefinition){
                if(!empty($fieldName) && $fieldName == 'participant_id'){
                    foreach ($filterDefinition as $operator => $ids) {
                        if (!in_array($operator, array('$in', '$not_in')) || empty($ids) || !is_array($ids))
                            continue;
                        $subQuery = new SugarQuery();
                        $subQuery->from(BeanFactory::getBean('PX_NeedToDoViewerComments'));
                        $commentUsersJoinA = $subQuery->join('dialog_users', array('joinType' => 'INNER'))->joinName();
                        $commentUsersJoinM2MA = $commentUsersJoinA . '_m2m';
                        $subQuery->where()->in($commentUsersJoinM2MA.'.user_id', $ids);
                        $subQuery->where()->equals($commentUsersJoinM2MA.'.type', 'author');
                        $subQuery->select(array('id'));

                        if($operator == '$in'){
                            $query->where()->in('id', $subQuery);
                        }else{
                            $query->where()->notIn('id', $subQuery);
                        }

                    }

                    unset($filterList[$key][$fieldName]);
                }
            }
        }
    }

    protected function applyMentionedFilter(&$filterList, SugarQuery $query){
        foreach($filterList as $key => $filter){
            foreach($filter as $fieldName => $filterDefinition){
                if(!empty($fieldName) && $fieldName == 'mentioned_id'){
                    foreach ($filterDefinition as $operator => $ids) {
                        if (!in_array($operator, array('$in', '$not_in')) || empty($ids) || !is_array($ids))
                            continue;
                        $subQuery = new SugarQuery();
                        $subQuery->from(BeanFactory::getBean('PX_NeedToDoViewerComments'));
                        $commentUsersJoinA = $subQuery->join('dialog_users', array('joinType' => 'INNER'))->joinName();
                        $commentUsersJoinM2MA = $commentUsersJoinA . '_m2m';
                        $subQuery->where()->in($commentUsersJoinM2MA.'.user_id', $ids);
                        $subQuery->where()->equals($commentUsersJoinM2MA.'.type', 'mentioned');
                        $subQuery->select(array('id'));

                        if($operator == '$in'){
                            $query->where()->in('id', $subQuery);
                        }else{
                            $query->where()->notIn('id', $subQuery);
                        }

                    }

                    unset($filterList[$key][$fieldName]);
                }
            }
        }
    }

    protected function applyNewMessagesFilter(&$filterList, SugarQuery $query){
        foreach($filterList as $key => $filter){
            foreach($filter as $fieldName => $filterDefinition){
                if(!empty($fieldName) && $fieldName == 'has_new_messages_id'){
                    foreach ($filterDefinition as $operator => $ids) {
                        if (!in_array($operator, array('$in', '$not_in')) || empty($ids) || !is_array($ids))
                            continue;
                        $subQuery = new SugarQuery();
                        $subQuery->from(BeanFactory::getBean('PX_NeedToDoViewerComments'));
                        $commentUsersJoinA = $subQuery->join('dialog_users', array('joinType' => 'INNER'))->joinName();
                        $commentUsersJoinM2MA = $commentUsersJoinA . '_m2m';
                        $subQuery->where()->in($commentUsersJoinM2MA.'.user_id', $ids);
                        $subQuery->where()->equals($commentUsersJoinM2MA.'.state', 'new');
                        $subQuery->select(array('id'));

                        if($operator == '$in'){
                            $query->where()->in('id', $subQuery);
                        }else{
                            $query->where()->notIn('id', $subQuery);
                        }

                    }

                    unset($filterList[$key][$fieldName]);
                }
            }
        }
    }

    protected function applyMyNewMessagesFilter(&$filterList, SugarQuery $query){
        foreach($filterList as $key => $filter){
            foreach($filter as $fieldName => $filterDefinition){
                if(!empty($fieldName) && $fieldName == '$my_new_messages'){
                    global $current_user;

                    $subQuery = new SugarQuery();
                    $subQuery->from(BeanFactory::getBean('PX_NeedToDoViewerComments'));
                    $commentUsersJoinA = $subQuery->join('dialog_users', array('joinType' => 'INNER'))->joinName();
                    $commentUsersJoinM2MA = $commentUsersJoinA . '_m2m';
                    $subQuery->where()->equals($commentUsersJoinM2MA.'.state', 'new');
                    $subQuery->where()->equals($commentUsersJoinM2MA.'.user_id', $current_user->id);
                    $subQuery->select(array('id'));
                    $query->where()->in('id', $subQuery);

                    unset($filterList[$key][$fieldName]);
                }
            }
        }
    }

    protected function applyMySubscriptionsFilter(&$filterList, SugarQuery $query){
        foreach($filterList as $key => $filter){
            foreach($filter as $fieldName => $filterDefinition){
                if(!empty($fieldName) && $fieldName == 'is_subscribed_to_id'){
                    foreach ($filterDefinition as $operator => $ids) {
                        if (!in_array($operator, array('$in', '$not_in')) || empty($ids) || !is_array($ids))
                            continue;

                        $subscribedJTA = 'subscribed_users_'.$key.'_'.$operator;
                        $subscribedTableSQL = "(SELECT id, created_by, parent_type, parent_id from subscriptions WHERE deleted=0)";
                        $query->join[$subscribedJTA] = $query->joinTable($subscribedTableSQL, array('alias' => $subscribedJTA, 'joinType' => 'LEFT'));
                        $query->join[$subscribedJTA]->on()->equalsField($subscribedJTA . '.parent_id', 'px_needtodoviewercomments.id')
                            ->equals($subscribedJTA . '.parent_type', 'PX_NeedToDoViewerComments');
                        if($operator == '$in') {
                            $query->where()->in($subscribedJTA . '.created_by', $ids);
                        }else{
                            $query->where()->notIn($subscribedJTA . '.created_by', $ids);
                        }
                        unset($filterList[$key][$fieldName]);

                    }
                }
            }
        }
    }

    protected function applySubscriptionFilter(&$filterList, SugarQuery $query){
        foreach($filterList as $key => $filter){
            foreach($filter as $fieldName => $filterDefinition){
                if(!empty($fieldName) && $fieldName == '$my_subscriptions'){
                    global $current_user;
                    $query->where()->equals('subscribed_users.created_by', $current_user->id);
                    unset($filterList[$key][$fieldName]);
                }
            }
        }
    }

    protected function applyRelevantToFilter(&$filterList, SugarQuery $query){
        foreach($filterList as $key => $filter){
            foreach($filter as $fieldName => $filterDefinition){
                if(!empty($fieldName) && $fieldName == 'is_relevant_to_id'){
                    foreach ($filterDefinition as $operator => $ids) {
                        if (!in_array($operator, array('$in', '$not_in')) || empty($ids) || !is_array($ids))
                            continue;

                        $aliasKey = 'is_relevant_to_id_' . $key . '_' . substr($operator, 1);
//                      $this->relevantFilterQuery($query, $operator, $ids, $aliasKey);
                        $subQuery = new SugarQuery();
                        $subQuery->from(BeanFactory::getBean('PX_NeedToDoViewerComments'));
                        $commentUsersJoinA = $subQuery->join('dialog_users', array('joinType' => 'LEFT'))->joinName();
                        $commentUsersJoinAM2M = $commentUsersJoinA.'_m2m';
//                      $SubscribedUsersJoinA = $subQuery->join('following_link', array('joinType' => 'LEFT'))->joinName();

                        $subscribedJTA = 'subscribed_' . $aliasKey;
                        $subscribedTableSQL = "(SELECT id, created_by, parent_type, parent_id from subscriptions WHERE deleted=0)";
                        $subQuery->join[$subscribedJTA] = $subQuery->joinTable($subscribedTableSQL, array('alias' => $subscribedJTA, 'joinType' => 'LEFT'));
                        $subQuery->join[$subscribedJTA]->on()->equalsField($subscribedJTA . '.parent_id', 'px_needtodoviewercomments.id')
                            ->equals($subscribedJTA . '.parent_type', 'PX_NeedToDoViewerComments');
//                            ->in($subscribedJTA . '.created_by', $ids);

                        $subQuery->select(array('id'));

                        $orWhere = $subQuery->where()->queryOr();
                        $andWhere = $orWhere->queryAnd();
                        $andWhere->in($commentUsersJoinA.'.id', $ids)->notEquals($commentUsersJoinAM2M.'.type', 'subscribed');
                        $orWhere->in($subscribedJTA . '.created_by', $ids);

                        if($operator == '$in') {
                            $query->where()->in('id', $subQuery);
                        }else{
                            $query->where()->notIn('id', $subQuery);
                        }

                        unset($filterList[$key][$fieldName]);

                    }
                }
            }
        }
    }

    protected function applyRelevantToMeFilter(&$filterList, SugarQuery $query){
        foreach($filterList as $key => $filter){
            foreach($filter as $fieldName => $filterDefinition){
                if(!empty($fieldName) && $fieldName == '$relevant_to_me'){
                    global $current_user;
                    $operator = '$in';
                    $ids = array($current_user->id);
//                    $this->relevantFilterQuery($query, $operator, $ids, $aliasKey);

                    $subQuery = new SugarQuery();
                    $subQuery->from(BeanFactory::getBean('PX_NeedToDoViewerComments'));
                    $commentUsersJoinA = $subQuery->join('dialog_users', array('joinType' => 'INNER'))->joinName();
                    $commentUsersJoinAM2M = $commentUsersJoinA.'_m2m';
                    $subQuery->select(array('id'));
                    $subQuery->where()->notEquals($commentUsersJoinAM2M.'.type', 'subscribed');
                    $subQuery->where()->in($commentUsersJoinA.'.id', $ids);


                    $orWhere = $query->where()->queryOr();
                    $orWhere->in('id', $subQuery);
                    $orWhere->equals('subscribed_users.created_by', $current_user->id);

                    unset($filterList[$key][$fieldName]);
                }
            }
        }
    }

    protected function applyNeedToDoEventsFilter(&$filterList, SugarQuery $query, $needtodoJoinAlias){
        foreach($filterList as $key => $filter){
            foreach($filter as $fieldName => $filterDefinition){
                if(!empty($fieldName) && in_array($fieldName,  array('filter_target_module', 'filter_object_module', 'filter_need_to_do_event_type_id'))){
                    foreach ($filterDefinition as $operator => $values) {
                        if (in_array($operator, array('$in', '$not_in')) && (empty($values) || !is_array($values)))
                            continue;

                        $fieldNameForQuery = $needtodoJoinAlias . '.';
                        if($fieldName == 'filter_target_module'){
                            $fieldNameForQuery .= 'parent_type';
                        }elseif($fieldName == 'filter_object_module'){
                            $fieldNameForQuery .= 'object_type';
                        }else{
                            $fieldNameForQuery .= 'event_id';
                        }

                        if($operator == '$in') {
                            $query->where()->in($fieldNameForQuery, $values);
                        }elseif($operator == '$not_in'){
                            $query->where()->notIn($fieldNameForQuery, $values);
                        }elseif($operator == '$empty'){
                            $query->where()->isEmpty($fieldNameForQuery);
                        }elseif($operator == '$not_empty'){
                            $query->where()->isNotEmpty($fieldNameForQuery);
                        }

                        unset($filterList[$key][$fieldName]);
                    }
                }
            }
        }
    }

    protected function formatResult(ServiceBase $api, array $args, SugarQuery $query)
    {
        global $locale, $current_user;

        $response = array();
        $data = $query->execute();

        $seed = BeanFactory::newBean('PX_NeedToDoViewerComments');
        $limit = $query->limit - 1;
        $count = count($data);
        if ($count > $limit) {
            $nextOffset = $query->offset + $limit;
            array_pop($data);
        } else {
            $nextOffset = -1;
        }

        $needToDoEventTypes = array();
        $needToDoEventBean = BeanFactory::getBean('PX_NeedToDoEvents');

        foreach ($data as $row) {
            $seed->populateFromRow($row, true);
            $record = $this->formatBean($api, $args, $seed);

            $record['needtodo_id'] = $row['needtodo_id'];
            $record['needtodo_name'] = $row['needtodo_name'];
            $record['needtodo_name_html'] = $row['needtodo_name_html'];
//            $record['needtodo_name_html'] = $needToDoEventBean->decodeNameHtmlField($row['needtodo_name_html']);
            $record['needtodo_event_type_id'] = $row['needtodo_event_type_id'];
            $record['needtodo_target_type'] = $row['needtodo_target_type'];
            $record['needtodo_target_id'] = $row['needtodo_target_id'];
            $record['following'] = !empty($row['subscription_id']);

            if(!empty($record['id'])){
                $countQuery = new SugarQuery();
                $countQuery->from(BeanFactory::getBean('PX_NeedToDoViewerComments'));

                $dialogUsers = $countQuery->join('dialog_users', array('joinType' => 'INNER'))->joinName();
                $dialogUsersM2M = $dialogUsers . '_m2m';
                $countQuery->where()->equals($dialogUsersM2M . '.state', 'new');
                $countQuery->where()->equals($dialogUsers.'.id', $current_user->id);
                $countQuery->where()->equals('px_needtodoviewercomments.id', $record['id']);
                $countQuery->select->selectReset()->setCountQuery();
                //$countQuery->order_by = null;
                $countQuery->limit = null;

                $countQueryResult = $countQuery->execute();
                $queryResult = reset($countQueryResult);
                $record['count_new_msg'] = $queryResult['record_count'];
                $record['has_new_messages'] = !empty($queryResult['record_count']) && $queryResult['record_count'] > 0 ? true : false;
            }

            $needToDoEventTypeName = '';
            $needToDoIcon = '';
            if(!empty($record['needtodo_event_type_id'])){
                $needToDoEventId = $record['needtodo_event_type_id'];
                if(!isset($needToDoEventTypes[$needToDoEventId])){
                    $needToDoEventTypes[$needToDoEventId] = BeanFactory::retrieveBean('PX_NeedToDoEventTypes', $needToDoEventId);
                }
                $eventBean = $needToDoEventTypes[$needToDoEventId];
                if(!empty($eventBean)) {
                    $needToDoEventTypeName = $eventBean->name;
                    $needToDoIcon = !empty($eventBean->icon);
                }
            }
            $record['needtodo_event_name'] = $needToDoEventTypeName;
            $record['needtodo_event_icon'] = $needToDoIcon;

            $needToDoEventTargetName = $needToDoEventBean->getRelatedEntityName($record['needtodo_target_type'], $record['needtodo_target_id']);;
            $record['needtodo_target_name'] = $needToDoEventTargetName;
            $response['records'][] = $record;
        }
        $response['next_offset'] = $nextOffset;
        $response['args'] = $args;
        return $response;
    }

//    protected function formatBean(ServiceBase $api, $args, SugarBean $bean, array $options = array()) {
    protected function formatBean(ServiceBase $api, array $args, SugarBean $bean, array $options = array()){
        $data = parent::formatBean($api, $args, $bean, $options);
        return $data;
    }

    public function CollaborationDialogHistory(ServiceBase $api, array $args){
        global $current_user;
        $query = new SugarQuery();
        $query->from(BeanFactory::getBean('PX_NeedToDoViewerComments'));
        $authorJoinA = $query->join('created_by_link', array('joinType' => 'LEFT'))->joinName();
        $dialogId = !empty($args['dialogId']) ? $args['dialogId'] : 'empty_parameter';
        $orWhere = $query->where()->queryOr();
        $orWhere->equals('id', $dialogId);
        $subAndWhere = $orWhere->queryAnd();
        $subAndWhere->equals('parent_id', $dialogId)->equals('parent_type', 'PX_NeedToDoViewerComments');
        $query->select(array('id', 'name', 'data', 'date_entered', 'created_by'));
        $query->select()->addField($authorJoinA.'.salutation', array('alias' => 'author_salutation'));
        $query->select()->addField($authorJoinA.'.first_name', array('alias' => 'author_first_name'));
        $query->select()->addField($authorJoinA.'.last_name', array('alias' => 'author_last_name'));
        $query->orderBy('date_entered', 'ASC');

        $data = $query->execute();

        $ids = array();
        foreach($data as $item) $ids[] = $item['id'];

        $stateQuery = new SugarQuery();
        $stateQuery->from(BeanFactory::getBean('PX_NeedToDoViewerComments'));
        $commentUsersJoinA = $stateQuery->join('comments_users', array('joinType' => 'INNER'))->joinName();
        $commentUsersJoinM2MA = $commentUsersJoinA . '_m2m';
        $stateQuery->where()->equals($commentUsersJoinM2MA . '.user_id', $current_user->id);
        $stateQuery->where()->in('id', $ids);
        $stateQuery->select(array('id'));
        $stateQuery->select()->addField($commentUsersJoinM2MA.'.state', array('alias' => 'message_state'));

        $stateData = array();
        $stateQueryResult = $stateQuery->execute();
        foreach($stateQueryResult as $stateItem){
//            $GLOBALS['log']->fatal('$stateQuery->compile()->getSQL(): ' . print_r($stateQuery->compile()->getSQL(), true));
//            $GLOBALS['log']->fatal('$stateQuery->compile()->getParameters(): ' . print_r($stateQuery->compile()->getParameters(), true));
            $stateData[$stateItem['id']] = $stateItem['message_state'];
        }
//        $GLOBALS['log']->fatal('$stateQueryResult: ' . print_r($stateQueryResult, true));

        global $locale, $timedate, $current_user;
        foreach($data as $key => $item){
            $data[$key]['date_entered'] = $timedate->to_display_date_time($item['date_entered'], true, true, $current_user);
            $data[$key]['data'] = json_decode(htmlspecialchars_decode($item['data']), true);
            $data[$key]['is_new'] = !empty($stateData[$item['id']]) && $stateData[$item['id']] == 'new' ?  1 : 0;

            if(!empty($item['created_by'])){
                $authorData = array(
                    'salutation' => $item['author_salutation'],
                    'first_name' => $item['author_first_name'],
                    'last_name' => $item['author_last_name']
                );
                $data[$key]['author_name'] = $locale->formatName('Users', $authorData);
            }
        }

        return array(
            'records' => $data
        );
    }

    public function CollaborationReadMessage(ServiceBase $api, array $args){

        global $current_user, $timedate;
        $currentDatetime = $timedate->nowDb();
        $userId = $current_user->id;
        $messageId = $args['messageId'];
        $messageBean = BeanFactory::retrieveBean('PX_NeedToDoViewerComments');
        if(!empty($messageId)) {
            $sql = "UPDATE px_needtodoviewercomments_users SET state='read', date_change_state='$currentDatetime' WHERE comment_id='$messageId' AND user_id='$userId'";
            $messageBean->db->query($sql);
        }
        return array('success' => true);
    }

    public function CollaborationMessageStatistics(ServiceBase $api, array $args){
        global $current_user, $timedate;
        $userId = $current_user->id;
        $messageId = $args['messageId'];
        $messageBean = BeanFactory::retrieveBean('PX_NeedToDoViewerComments', $messageId);
        $data = array(
            'readMessages' => 0,
            'totalMessages' => 0,
            'participants' => array()
        );
        if(!empty($messageBean)) {
            $query = new SugarQuery();
            $query->from(BeanFactory::getBean('PX_NeedToDoViewerComments'));
            $messageUsers = $query->join('comments_users', array('joinType' => 'INNER'))->joinName();
            $messageUsersM2M = $messageUsers . '_m2m';
            $query->select(array('created_by'));
            $query->select()->addField($messageUsers . '.id', array('alias' => 'participant_id'));
            $query->select()->addField($messageUsers . '.salutation', array('alias' => 'participant_salutation'));
            $query->select()->addField($messageUsers . '.first_name', array('alias' => 'participant_first_name'));
            $query->select()->addField($messageUsers . '.last_name', array('alias' => 'participant_last_name'));
            $query->select()->addField($messageUsersM2M . '.state', array('alias' => 'message_state'));
            $query->select()->addField($messageUsersM2M . '.date_change_state', array('alias' => 'message_date_change_state'));
            $query->where()->equals('id', $messageId);
            $query->where()->notEquals($messageUsersM2M . '.user_id', $userId);
            $query->orderBy($messageUsersM2M . '.date_change_state', 'DESC');

            global $locale;
            foreach($query->execute() as $item){
                if($item['created_by'] == $item['participant_id'])
                    continue;

                $data['totalMessages']++;
                if($item['message_state'] == 'read'){
                    $data['readMessages']++;
                }

                if(!empty($item['participant_id'])){
                    $participantData = array(
                        'salutation' => $item['participant_salutation'],
                        'first_name' => $item['participant_first_name'],
                        'last_name' => $item['participant_last_name']
                    );
                    $data['participants'][] = array(
                        'participant_id' => $item['participant_id'],
                        'participant_full_name' => $locale->formatName('Users', $participantData),
                        'message_state' => $item['message_state'],
                        'message_read_date' => $item['message_date_change_state'],
                    );
                }
            }
        }
        return $data;
    }
}