<?php

require_once('modules/PX_NeedToDoEventTypes/PX_NeedToDoEventTypes_sugar.php');

class PX_NeedToDoEventTypes extends PX_NeedToDoEventTypes_sugar
{
    const ENABLED_STATUS = "Enabled";

    public $lbTemplateBean;

    protected static $userNames = array();

    public static function getTargetModuleList()
    {
        $targetModuleFilter = new \Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\Helpers\TargetModuleListFilter();
        return $targetModuleFilter->getModuleList();
    }

    public static function getTargetAttributeList($targetModule = '')
    {
//        if(empty($targetModule) && !empty($this->target_module)){
//
//        }
        $targetAttributeFilter = new \Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\Helpers\TargetAttributeListFilter();
        return $targetAttributeFilter->getAttributeList($targetModule);
    }

    public static function getTargetAttributeValues($targetModule = '')
    {
        $templateArgumentsListFilter = new \Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\Helpers\TemplateArgumentsListFilter();
        return $templateArgumentsListFilter->getAttributeList($targetModule);
    }

    public static function getLBTemplateArguments($lbTemplateID = '')
    {
        $options = array();
        $lbTemplatesArguments = BeanFactory::retrieveBean('PX_FlowchartTemplates', $lbTemplateID);
        if(!empty($lbTemplatesArguments) && !empty($lbTemplatesArguments->context)){
            if(is_string($lbTemplatesArguments->context)){
                $context = json_decode($lbTemplatesArguments->context, true);
                foreach($context as $item){
                    $options[$item['name']] = $item['label'];
                }
            }
        }
        return $options;
    }

    function save($check_notify = false)
    {
        if (!empty($this->uploaded_icon_id)) {
            $uploadFile = new UploadFile();
            $uploadFile->temp_file_location = $uploadFile->get_upload_path('tmp/' . $this->uploaded_icon_id);

            $this->icon = base64_encode($uploadFile->get_file_contents());
        }

        $this->setControlDateType();
        $this->processTargetEntityFilterBeforeSave();
        $this->notify_users = $this->processNotifyUserListBeforeSave($this->notify_users);

        $this->pruneEventsHistory();

//        $GLOBALS['log']->fatal('$this->notify_users: ' . print_r($this->notify_users, true));

        $id = parent::save($check_notify);
        return $id;
    }

    protected function pruneEventsHistory(){
        if(empty($this->id) || !empty($this->new_with_id)){
            return false;
        }

        if(!$this->pruneEvents()){
            $this->pruneEventsRelatedList();
            $this->pruneEventsNotifyData();
        }
    }

    protected function pruneEvents(){
        $historyCleaner = new \Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\PX_NeedToDoEvents\helpers\HistoryCleaner();
        if(
            $this->target_module != $this->fetched_row['target_module'] ||
            $this->target_attribute_name != $this->fetched_row['target_attribute_name'] ||
            $this->target_entity_filter != $this->fetched_row['target_entity_filter']
        ) {
            $historyCleaner->cleanEventsHistory($this->id);
            return true;
        }
        return false;
    }

    protected function pruneEventsRelatedList(){
        $historyCleaner = new \Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\PX_NeedToDoEvents\helpers\HistoryCleaner();
        if($this->target_entity_parent_list != $this->fetched_row['target_entity_parent_list']) {
            $historyCleaner->cleanRelatedParentsData($this->id);
            return true;
        }
        return false;
    }

    protected function pruneEventsNotifyData(){
        $historyCleaner = new \Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\PX_NeedToDoEvents\helpers\HistoryCleaner();
        $fetchedNotifyUsersList = $this->processNotifyUserListBeforeSave($this->fetched_row['notify_users']);
        if(
            $this->notify_if_overdue && $this->notify_if_overdue != $this->fetched_row['notify_if_overdue']
        ){
            $historyCleaner->cleanNotifyUsersData($this->id);
            return true;
        }else if(
            $this->notify_beforehand &&
            (
                $this->notify_beforehand != $this->fetched_row['notify_beforehand'] ||
                $this->notify_beforehand_units != $this->fetched_row['notify_beforehand_units'] ||
                $this->notify_beforehand_duration != $this->fetched_row['notify_beforehand_duration']
            )
        ){
            $historyCleaner->cleanNotifyUsersData($this->id);
            return true;
        }else if(
            ($this->notify_if_overdue || $this->notify_beforehand) &&
            (
                $this->notify_users != $fetchedNotifyUsersList ||
                $this->notify_assigned_user != $this->fetched_row['notify_assigned_user']
            )
        ){
            $historyCleaner->cleanNotifyUsersData($this->id);
            return true;
        }
        return false;
    }

    private function setControlDateType(){
        $this->target_attribute_type = $this->getTargetAttributeType($this->target_module, $this->target_attribute_name);
    }

    private function processTargetEntityFilterBeforeSave(){
        if(!empty($this->target_entity_filter) && is_string($this->target_entity_filter)) {
            $this->target_entity_filter = json_decode(htmlspecialchars_decode($this->target_entity_filter), true);
            if (json_last_error() !== JSON_ERROR_NONE)
                $this->target_entity_filter = array();
        }

        if(!is_string($this->target_entity_filter)){
            $this->target_entity_filter = json_encode($this->target_entity_filter);
        }
    }

    private function processNotifyUserListBeforeSave($notifyUsers){
        if(empty($notifyUsers))
            return;

        $notifyUsersDecoded = $this->jsonDecodeToArray($notifyUsers);
        $notifyUsers = array();
        if(!empty($notifyUsersDecoded) && is_array($notifyUsersDecoded)) {
            foreach($notifyUsersDecoded as $userData){
                $notifyUsers[] = array(
                    'id' => $userData['id'],
                    'key' => $userData['key'],
                );
            }
        }

        return $this->jsonEncodeFromArray($notifyUsers);
    }

    public function decodeNotifyUsers(){
        return $this->jsonDecodeToArray($this->notify_users);
    }

    private function jsonDecodeToArray($value){
        if(empty($value))
            return array();

        if(is_string($value)){
            $value = json_decode($value, true);
            if(json_last_error() != JSON_ERROR_NONE){
                $value = array();
            }
        }

        return $value;
    }

    private function jsonEncodeFromArray($field){
        if(empty($field) ) {
            return '';
        }elseif(is_string($field)) {
            return $field;
        }

        return json_encode($field);
    }

    function fetchFromQuery(SugarQuery $query, array $fields = array(), array $options = array())
    {
        $beans = array();

        /** @var SugarFieldBase[] $secondaryFields */
        list($queryFields, $additionalQueryFields, $secondaryFields) = $this->getQueryFields($query, $fields, $options);

        $query->select($queryFields);

        if ($query->shouldFetchErasedFields()) {
            $this->retrieve_erased_fields = true;
        }

        if ($this->queryProducesDuplicates($query)) {
            $this->fixQuery($query);
        }

        $this->call_custom_logic('before_fetch_query', array('query' => $query, 'fields' => $fields));

        $rows = $query->execute();

        $additionalRows = $this->fetchFromAdditionalQuery($additionalQueryFields, $rows);

        $rawRows = array();
        foreach ($rows as $row) {
            if (isset($options['beanList'][$row['id']])) {
                $bean = $options['beanList'][$row['id']];
            } else {
                $bean = $this->getCleanCopy();
            }
            if (count($additionalRows) && array_key_exists($row['id'], $additionalRows)) {
                $row = array_merge($row, $additionalRows[$row['id']]);
            }
            //true parameter below tells populate to perform conversions on row data
            $bean->fetched_row = $bean->populateFromRow($row, true);
            $this->populateFetchedEmail();
            $bean->call_custom_logic("process_record");
            $beans[$bean->id] = $bean;
            $rawRows[$bean->id] = $row;
        }

        if (!isset($options['skipSecondaryQuery'])
            || $options['skipSecondaryQuery'] == false) {
            foreach ($secondaryFields as $fieldName => $sugarField) {
                $sugarField->runSecondaryQuery($fieldName, $this, $beans);
            }
        }

        $this->call_custom_logic('after_fetch_query', array('beans' => $beans, 'fields' => $fields, 'rows' => $rawRows));

        if (!empty($options['compensateDistinct'])) {
            $beans['_distinctCompensation'] = $this->computeDistinctCompensation($rows, $beans);
        }

        if (!empty($options['returnRawRows'])) {
            $beans['_rows'] = $rawRows;
        }

        return $beans;
    }

    function retrieve($id = '-1', $encode = true, $deleted = true)
    {
        $fetched = parent::retrieve($id, $encode, $deleted);
//        $this->target_attribute_type = $this->getTargetAttributeType($this->target_module, $this->target_attribute_name);
        $this->notify_users = $this->getNotifyUsersNames($this->notify_users);
        return $fetched;
    }

    public function populateFromRow(array $row, $convert = false)
    {
        $row = parent::populateFromRow($row, $convert);

//        $this->target_attribute_type = $row['target_attribute_type'] = $this->getTargetAttributeType($row['target_module'], $row['target_attribute_name']);
        $this->notify_users = $row['notify_users'] = $this->getNotifyUsersNames($row['notify_users']);

        return $row;
    }

    public function getTargetAttributeType($targetModule, $targetAttributeName){
        $targetAttributeFilter = new \Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\Helpers\TargetAttributeListFilter();
        $type = $targetAttributeFilter->getAttributeType($targetModule, $targetAttributeName);
        return $type;
    }

    public function getNotifyUsersNames($notifyUsers){
        $notifyUsers = $this->jsonDecodeToArray($notifyUsers);
        if(empty($notifyUsers)){
            return '';
        }
        $notifyUserIds = array();
        foreach($notifyUsers as $notifyUser){
            $userId = $notifyUser['id'];
            if(!isset(self::$userNames[$userId])) {
                $notifyUserIds[$userId] = $userId;
            }
        }

//        $GLOBALS['log']->fatal('$notifyUserIds: ' . print_r($notifyUserIds, true));

        if(!empty($notifyUserIds)){
            $query = new SugarQuery();
            $query->from(BeanFactory::getBean('Users'));
            $query->select(array('id', 'first_name', 'last_name'));
            $query->where()->in('id', $notifyUserIds);
            $queryResult = $query->execute();

//            $GLOBALS['log']->fatal('$queryResult: ' . print_r($queryResult, true));

            global $locale;
            if (!empty($queryResult)) {
                foreach($queryResult as $userName) {
                    self::$userNames[$userName['id']] = $locale->formatName('Users', $userName);
                    unset($notifyUserIds[$userName['id']]);
                }
            }

//            $GLOBALS['log']->fatal('self::$userNames: ' . print_r(self::$userNames, true));
            foreach($notifyUserIds as $notifyUserId){
                self::$userNames[$notifyUserId] = '';
            }
        }

//        $GLOBALS['log']->fatal('self::$userNames: ' . print_r(self::$userNames, true));

        foreach($notifyUsers as $key => $notifyUser){
            if(isset(self::$userNames[$notifyUser['id']])){
                $notifyUsers[$key]['name'] = self::$userNames[$notifyUser['id']];
            }
        }
//        $GLOBALS['log']->fatal('$notifyUsers: ' . print_r($notifyUsers, true));
        return $this->jsonEncodeFromArray($notifyUsers);
    }

    public function getParentRelatedNames($targetEntityParentList){
        if(empty($targetEntityParentList))
            return array();

        if(is_string($targetEntityParentList)){
            $targetEntityParentList = json_decode($targetEntityParentList, true);
            if(json_last_error() == JSON_ERROR_NONE && !empty($targetEntityParentList)){
                return $targetEntityParentList;
            }
        }else if(is_array($targetEntityParentList)){
            return $targetEntityParentList;
        }
        return array();
    }

}