<?php

require_once('modules/PX_NeedToDoEvents/PX_NeedToDoEvents_sugar.php');
require_once('custom/PxRoutines/NeedToDoViewer/PX_NeedToDoEvents/visibility/NTDVSecurityInheritance.php');

class PX_NeedToDoEvents extends PX_NeedToDoEvents_sugar
{
    protected static $StackedRelatedEntities = array();
    protected static $needToDoEventTypesStack = array();
    protected static $parentsListLoadedStack = array();

    protected $objectEntity;

    function save($check_notify = false)
    {

        if(empty($this->id)){
            $this->id = Sugarcrm\Sugarcrm\Util\Uuid::uuid1();
            $this->new_with_id = true;
        }

        $this->inheritParentSecurity();

        $this->generateNameHtmlFromLBTemplate();
        $this->generateNameFromNameHtml();
        $this->setNeedToDoControlDate();
        $this->setNeedToDoNotifyDate();
        $this->setParentList();
        $this->setNotifiedUsers();
        $this->addObjectRecordToParentList();

        $id = parent::save($check_notify);
        return $id;
    }

    function retrieve($id='-1', $encode=true,$deleted=true){
        try{
            require_once 'custom/PxRoutines/NeedToDoViewer/Package/License/Control.php';
            $control = new \Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\Package\License\Control();
            $control->checkLicenseForCurrentUser();
        }catch(Exception $e){
            return null;
        }
        return parent::retrieve($id, $encode, $deleted);
    }

    private function generateNameFromNameHtml(){
        $this->name = '';
        if(!empty($this->name_html)){
            $this->name = html_entity_decode(htmlspecialchars_decode(strip_tags($this->name_html)));
        }
        if(empty($this->name)){
            $eventTypeBean = $this->getEventTypeBean();
            if(!empty($eventTypeBean) && !empty($eventTypeBean->name)){
                $this->name = $eventTypeBean->name;
            }
        }
    }

    private function generateNameHtmlFromLBTemplate()
    {
        $this->name_html = '';
        $eventTypeBean = $this->getEventTypeBean();
        if(!$this->isTypeBeanValidForTemplateGeneration($eventTypeBean))
            return;

        $lbTemplateBean = $this->getLBTemplateBean($eventTypeBean);
        if(!$this->isLBTemplateValidForTemplateGeneration($lbTemplateBean))
            return;

        $targetEntityBean = $this->getTargetEntityBean();
        $attributes = $this->getAttributes($eventTypeBean, $targetEntityBean);

        if(class_exists('\\Sugarcrm\\Sugarcrm\\custom\\PxRoutines\\LBExtensions\\Templates\\Helpers\\TemplateProcessor')) {
            $templateEngine = new \Sugarcrm\Sugarcrm\custom\PxRoutines\LBExtensions\Templates\Helpers\TemplateProcessor($lbTemplateBean, $attributes);
            try {
                $this->name_html = $templateEngine->process();
            } catch (Exception $e) {
                $this->name_html = '';
            }
        }
    }

    private function setNeedToDoControlDate(){
        $eventTypeBean = $this->getEventTypeBean();
        if(!$this->isTypeBeanValidForControlDateIdentification($eventTypeBean)) {
            return;
        }

        if($eventTypeBean->target_attribute_type == 'date'){
            global $timedate;
            $gmtTimezone = new DateTimeZone('GMT');
            $date = SugarDateTime::createFromFormat($timedate->get_db_date_time_format(), $this->control_date, $gmtTimezone);

            if (empty($date)) {
                $time = '00:00:00';

                $date = SugarDateTime::createFromFormat($timedate->get_db_date_time_format(),
                    $timedate->merge_date_time($this->control_date, $time), $gmtTimezone);
            }

            if (empty($date)) {
                $this->control_date = '';
            }else{
                $this->control_date = $date->asDb();
            }

        }
    }

    private function setNeedToDoNotifyDate(){
        $eventTypeBean = $this->getEventTypeBean();
        if(!$this->typeBeanContainsNotificationDateIdentification($eventTypeBean)) {
            return;
        }

        $this->notify_date = $this->getNotifyBeforehandDate($eventTypeBean);
    }

    private function getNotifyBeforehandDate($eventTypeBean){
        global $timedate;

        $lifetimeUnits = $eventTypeBean->notify_beforehand_units;
        $lifetimeDuration = $eventTypeBean->notify_beforehand_duration;
        $interval = $this->getIntervalKey($lifetimeUnits, $lifetimeDuration);
        $controlDate = $this->getControlDateForNotifyDate();
        $notifyDate = $timedate->fromDb($controlDate);

//        $GLOBALS['log']->fatal('$controlDate: ' . print_r($controlDate, true));
//        $GLOBALS['log']->fatal('$interval: ' . print_r($interval, true));

        if(empty($notifyDate)){
            $notifyDate = $timedate->fromUser($controlDate);
        }
        if(!empty($notifyDate)){
//            $GLOBALS['log']->fatal('$notifyDate: ' . print_r($notifyDate, true));
            try {
                $notifyDate->sub(new DateInterval($interval));
                $resultDate = $notifyDate->asDb();
//                $GLOBALS['log']->fatal('$resultDate: ' . print_r($resultDate, true));
                return $resultDate;
            }catch(Exception $e){
//                $GLOBALS['log']->fatal('$e->getMessage(): ' . print_r($e->getMessage(), true));
            }
        }

        return null;
    }

    private function getIntervalKey($lifetimeUnits, $lifetimeDuration){
        $number = abs($lifetimeDuration);
        $interval = 'P';

        if($lifetimeUnits == 'Years')
            $interval .= $number . 'Y'; elseif($lifetimeUnits == 'Months')
            $interval .= $number . 'M';
        elseif($lifetimeUnits == 'Days')
            $interval .= $number . 'D';
        elseif($lifetimeUnits == 'Weeks')
            $interval .= ($number * 7) . 'D';
        elseif($lifetimeUnits == 'Hours')
            $interval .= 'T'.$number . 'H';
        elseif($lifetimeUnits == 'Minutes')
            $interval .= 'T'.$number . 'M';

        return $interval;
    }

    private function getControlDateForNotifyDate(){
        $eventTypeBean = $this->getEventTypeBean();
        if(!$this->isTypeBeanValidForControlDateIdentification($eventTypeBean)) {
            return $this->control_date;
        }

        if($eventTypeBean->target_attribute_type == 'date'){
            global $timedate;
            $controlDate = $timedate->fromDb($this->control_date);
            if(empty($controlDate)){
                $controlDate = $timedate->fromUser($this->control_date);
            }
            if(!empty($controlDate)){
                $controlDate = $controlDate->asDbDate(false);
                $time =  '00:00:00';
                $gmtTimezone = new DateTimeZone('GMT');

                return SugarDateTime::createFromFormat($timedate->get_db_date_time_format(),
                    $timedate->merge_date_time($controlDate, $time), $gmtTimezone);
            }
        }
        return $this->control_date;
    }

    public function getControlDateForDateFormat($controlDate, $controlDateType){
        if($controlDateType == 'date'){
            global $timedate;
            $time =  '23:59:59';
            $gmtTimezone = new DateTimeZone('GMT');

            $date = SugarDateTime::createFromFormat($timedate->get_db_date_time_format(),
                $timedate->merge_date_time($controlDate, $time), $gmtTimezone);

            if (empty($date)) {
                return '';
            }else{
                return $date->asDb();
            }
        }
        return $controlDate;
    }

    public function refreshParentsList(){

    }

    private function setParentList(){
        $this->cleanUpOldParentList();
        $eventTypeBean = $this->getEventTypeBean();
        if(!$this->isTypeBeanValidForParentListComposition($eventTypeBean)) {
            return;
        }

        $targetEntityBean = $this->getTargetEntityBean();

        $linkNames = $eventTypeBean->getParentRelatedNames($eventTypeBean->target_entity_parent_list);

        $parentRelatedListRetriever = new \Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\Helpers\ParentRelatedListRetriever();
        $parentRelatedListRetriever->setRelateParentList($this, $targetEntityBean, $linkNames);
    }

    private function addObjectRecordToParentList(){
        $ntdvRelates = BeanFactory::getBean('PX_NeedToDoRelates');
        $ntdvRelates->parent_type = $this->object_type;
        $ntdvRelates->parent_id = $this->object_id;
        $ntdvRelates->relate_type = 'Object';
        $ntdvRelates->event_id = $this->id;
        $ntdvRelates->link_name = '';
        $ntdvRelates->save();
    }

    private function cleanUpOldParentList(){
        $parentRelatedList = new \Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\Helpers\ParentRelatedListRetriever();
        $parentRelatedList->deleteOldLinks($this->id);
    }

    private function setNotifiedUsers(){
        $eventTypeBean = $this->getEventTypeBean();
        $notifyUserList = new \Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\PX_NeedToDoEvents\helpers\NotifyUsersList($eventTypeBean, $this);
        $notifyUserList->setUsersForNotification();
    }

    protected function inheritParentSecurity(){
        $security = new NTDVSecurityInheritance($this);
        $security->inherit();
    }

    public function setNeedToDoEventTypeBean($eventTypeBean){
        self::$needToDoEventTypesStack[$eventTypeBean->id] = $eventTypeBean;
    }

    private function getEventTypeBean($needToDoEventTypeId = ''){
        $needToDoEventTypeId = !empty($needToDoEventTypeId) ? $needToDoEventTypeId : $this->event_type_id;
        if(empty($needToDoEventTypeId))
            return;

        if(isset(self::$needToDoEventTypesStack[$needToDoEventTypeId])){
            return self::$needToDoEventTypesStack[$needToDoEventTypeId];
        }
        $needToDoEventTypeBean = BeanFactory::retrieveBean('PX_NeedToDoEventTypes', $needToDoEventTypeId, array('disable_row_level_security' => true));
        if(!empty($needToDoEventTypeBean)){
            self::$needToDoEventTypesStack[$needToDoEventTypeId] = $needToDoEventTypeBean;
        }else{
            self::$needToDoEventTypesStack[$needToDoEventTypeId] = null;
        }

        return self::$needToDoEventTypesStack[$needToDoEventTypeId];
    }

    private function isTypeBeanValidForTemplateGeneration($needToDoEventTypeBean){
        return !empty($needToDoEventTypeBean) && !empty($needToDoEventTypeBean->lb_template_id);
    }

    private function isTypeBeanValidForControlDateIdentification($needToDoEventTypeBean){
        return !empty($needToDoEventTypeBean) && !empty($needToDoEventTypeBean->target_attribute_type);
    }

    private function typeBeanContainsNotificationDateIdentification($needToDoEventTypeBean){
        return !empty($needToDoEventTypeBean) && !empty($needToDoEventTypeBean->notify_beforehand) && !empty($needToDoEventTypeBean->notify_beforehand_units) && !empty($needToDoEventTypeBean->notify_beforehand_duration);
    }

    private function isTypeBeanValidForNotification($needToDoEventTypeBean){
        return !empty($needToDoEventTypeBean) && !empty($needToDoEventTypeBean->notify_assigned_user) && !empty($needToDoEventTypeBean->notify_users);
    }

    private function isTypeBeanValidForParentListComposition($needToDoEventTypeBean){
        if(empty($needToDoEventTypeBean)) {
            return false;
        }
        $parentRelatedNames = $needToDoEventTypeBean->getParentRelatedNames($needToDoEventTypeBean->target_entity_parent_list);
        return !empty($parentRelatedNames);
    }

    private function isLBTemplateValidForTemplateGeneration($lbTemplateBean){
        return !empty($lbTemplateBean) && !empty($lbTemplateBean->html_body);
    }

    private function getLBTemplateBean($needToDoEventTypeBean){
        if(!empty($needToDoEventTypeBean->lb_template_id)){
            if(!isset($needToDoEventTypeBean->lbTemplateBean)){
                $needToDoEventTypeBean->lbTemplateBean = BeanFactory::retrieveBean('PX_FlowchartTemplates', $needToDoEventTypeBean->lb_template_id, array('disable_row_level_security' => true));
            }
            return $needToDoEventTypeBean->lbTemplateBean;
        }
        return null;
    }

    public function getTargetEntityBean(){
        if(!empty($this->object_type) && !empty($this->object_id)){
            if(!isset($this->objectEntity)){
                $this->objectEntity = BeanFactory::retrieveBean($this->object_type, $this->object_id, array('disable_row_level_security' => true));
            }
            return $this->objectEntity;
        }
        return null;
    }

    private function getAttributes($eventTypeBean, $targetEntityBean){
        if(empty($eventTypeBean->lb_template_arguments))
            return array();

        $attributes = array();
        if(is_string($eventTypeBean->lb_template_arguments)) {
            $arguments = json_decode($eventTypeBean->lb_template_arguments, true);
        }else{
            $arguments = $eventTypeBean->lb_template_arguments;
        }

        if(!empty($arguments) && is_array($arguments)) {
            foreach ($arguments as $argument) {
                if (isset($argument['label']) && isset($argument['name'])) {
                    $fieldName = $argument['name'];
                    $attributes[$argument['label']] = $targetEntityBean->getFieldValue($fieldName);
                }
            }
        }

        return $attributes;
    }

    public function populateFromRow(array $row, $convert = false)
    {
//        $this->parents_list = array();
        $row = parent::populateFromRow($row, $convert);

        $this->populateObjectName();
        $this->event_status = $this->populateEventStatus();
        $this->parents_list = $this->populateParentsList();

        return $row;
    }

    private function populateObjectName(){
        if (!empty($this->objectEntity)) {
            if ($this->objectEntity instanceof Person)
                $object = $this->objectEntity->full_name;
            else
                $object = $this->objectEntity->name;
            self::$StackedRelatedEntities[$this->object_type][$this->object_id] = $this->object_name = $object;
        } else {
            $this->object_name = $this->getRelatedEntityName($this->object_type, $this->object_id);
        }
    }

    private function getControlDatetime(){
        global $timedate;

        $controlDate = $this->control_date;
        $controlDatetime = $timedate->fromDb($controlDate);
        if(empty($controlDatetime)){
            $controlDatetime = $timedate->fromUser($controlDate);
        }
        if(empty($controlDatetime)){
            $controlDatetime = $timedate->fromIso($controlDate);
        }
        return $controlDatetime;
    }

    private function populateEventStatus(){
        global $timedate;

        $controlDatetime = $this->getControlDatetime();
        if(empty($controlDatetime)){
            return '';
        }
        $controlDatetimeTimestamp = $controlDatetime->getTimestamp();

        if($this->control_date_type == 'date'){
            $currentDate = $timedate->nowDbDate();
            $gmtTimezone = new DateTimeZone('GMT');
            $overdueDateTimeToTimestamp = SugarDateTime::createFromFormat($timedate->get_db_date_time_format(),
                $timedate->merge_date_time($currentDate, '00:00:00'), $gmtTimezone)->getTimestamp();

            if($controlDatetimeTimestamp < $overdueDateTimeToTimestamp){
                return 'overdue';
            }

            $futureDateTimeFromTimestamp = SugarDateTime::createFromFormat($timedate->get_db_date_time_format(),
                $timedate->merge_date_time($currentDate, '23:59:59'), $gmtTimezone)->getTimestamp();

            if($controlDatetimeTimestamp >= $overdueDateTimeToTimestamp && $controlDatetimeTimestamp <= $futureDateTimeFromTimestamp){
                return 'current';
            }

            if($controlDatetimeTimestamp > $futureDateTimeFromTimestamp){
                return 'future';
            }
        }

        $currentDateTimeFrom = $timedate->getNow();
        $currentDateTimeFrom->sub(new DateInterval('PT15M'));
        $currentDateTimeFromTimestamp = $currentDateTimeFrom->getTimestamp();

        if($controlDatetimeTimestamp < $currentDateTimeFromTimestamp){
            return 'overdue';
        }

        $currentDateTimeTo = $timedate->getNow();
        $currentDateTimeTo->add(new DateInterval('PT1H'));
        $currentDateTimeToTimestamp = $currentDateTimeTo->getTimestamp();

        if($controlDatetimeTimestamp >= $currentDateTimeFromTimestamp && $controlDatetimeTimestamp <= $currentDateTimeToTimestamp){
            return 'current';
        }

        if($controlDatetimeTimestamp > $currentDateTimeToTimestamp){
            return 'future';
        }
    }

    public function populateParentsList($eventId = '', $eventTypeId = ''){

        $eventId = !empty($eventId) ? $eventId : $this->id;
        $eventTypeId = !empty($eventTypeId) ? $eventTypeId : $this->event_type_id;
        $parentsList = array();

        if(!isset(self::$parentsListLoadedStack[$eventId])) {
            $parentsMeta = $this->retrieveParentsMeta($eventId);
            $eventTypeLinksList = $this->getLinkNames($eventTypeId);

            foreach($parentsMeta as $parentModule => $links) {
                foreach($links as $linkName => $parentIds) {
                    $relatedData = $this->getRelatedEntityData($parentModule, $parentIds);
                    $linkLabel = $this->getLinkLabel($parentModule, $linkName, $eventTypeLinksList);

                    foreach($relatedData as $relatedItem) {
                        $parentsList[] = array(
                            'moduleName' => $parentModule,
                            'linkName' => $linkName,
                            'linkLabel' => $linkLabel,
                            'recordId' => $relatedItem['id'],
                            'recordName' => $relatedItem['name'],
                        );
                    }
                }
            }

            self::$parentsListLoadedStack[$eventId] = $parentsList;
        }else{
            $parentsList = self::$parentsListLoadedStack[$eventId];
        }

        return $parentsList;
    }

    private function getLinkLabel($moduleName, $linkName, $eventTypeLinksList){
        $label = '';
        if(is_array($eventTypeLinksList) && !empty($eventTypeLinksList)) {
            foreach ($eventTypeLinksList as $eventTypeLinksMeta) {
                if ($linkName == $eventTypeLinksMeta['linkLevel1'] && !empty($eventTypeLinksMeta['linkLabel'])) {
                    $label = $eventTypeLinksMeta['linkLabel'];
                    break;
                }
            }
        }
        if(empty($label)){
            $seed = BeanFactory::getBean($moduleName);
            if(!empty($seed) && !empty($seed->field_defs[$linkName]) && !empty($seed->field_defs[$linkName]['vname'])) {
                $vname = $seed->field_defs[$linkName]['vname'];
                $label = translate($vname, $moduleName);
            }
        }
        if(empty($label)){
            global $app_list_strings;
            if(!empty($app_list_strings['moduleList'][$moduleName]))
                $label = $app_list_strings['moduleList'][$moduleName];
            else
                $label = $moduleName;
        }
        return $label;
    }

    private function getLinkNames($eventTypeBeanId){
        $eventTypeBean = $this->getEventTypeBean($eventTypeBeanId);
        if(!$this->isTypeBeanValidForParentListComposition($eventTypeBean)) {
            return array();
        }
        return $eventTypeBean->getParentRelatedNames($eventTypeBean->target_entity_parent_list);
    }

    private function retrieveParentsMeta($eventId){
        $resultData = array();

        $query = new SugarQuery();
        $query->from(BeanFactory::getBean('PX_NeedToDoRelates'));
        $query->select(array('parent_type', 'parent_id', 'link_name'));
        $query->where()->equals('event_id', $eventId)
            ->equals('relate_type', 'Target');
        $queryResult = $query->execute();

        foreach($queryResult as $row){
            $moduleName = $row['parent_type'];
            $linkName = $row['link_name'];
            $parentId = $row['parent_id'];
            if(!empty($moduleName) && !empty($parentId) && !empty($linkName)) {
                $resultData[$moduleName][$linkName][] = $parentId;
            }
        }

//        $db = \DBManagerFactory::getInstance();
//        $sql = "SELECT parent_type, parent_id, link_name FROM px_ntdv_parents WHERE event_id='$eventId'";
//        $rs = $db->query($sql);
//        while($row = $db->fetchByAssoc($rs)){
//
//        }
        return $resultData;
    }

    public function getControlDateType($targetModule, $targetAttributeName){
        $targetAttributeFilter = new \Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\Helpers\TargetAttributeListFilter();
        return $targetAttributeFilter->getAttributeType($targetModule, $targetAttributeName);
    }

    public function formatControlDate($controlDateType, $controlDate){
        global $timedate;
        $controlDatetime = $timedate->fromDb($controlDate);
        if (empty($controlDatetime)) {
            $controlDatetime = $timedate->fromIso($controlDate);
        }
        if (!empty($controlDatetime)) {
            if($controlDateType == 'date') {
                $controlDate = $timedate->asIsoDate($controlDatetime);
            }else{
                $controlDate = $timedate->asIso($controlDatetime);
            }
            return $controlDate;
        }

        return $controlDate;
    }

    public function getRelatedEntityData($moduleName, $recordIds){
        $result = array();
        if (empty($moduleName) || empty($recordIds)) {
            return $result;
        }

        $searchRecordIds = is_array($recordIds) ? $recordIds : [$recordIds];
        foreach($recordIds as $recordId){
            if (!isset(self::$StackedRelatedEntities[$moduleName][$recordId])){
                $searchRecordIds[] = $recordId;
            }
        }

        if(!empty($searchRecordIds)) {
            $seed = BeanFactory::getBean($moduleName);
            $targets = array();

            if (!empty($seed)) {
                $query = new SugarQuery();
                $query->from($seed);
                $query->where()->in('id', $searchRecordIds);
                if ($seed instanceof Person) {
                    $query->select(array('id', 'salutation', 'first_name', 'last_name'));
                    $targetData = $query->execute();
                    global $locale;
                    if (!empty($targetData)) {
                        foreach($targetData as $targetItem) {
                            $targets[] = array(
                                'id' => $targetItem['id'],
                                'name' => $locale->formatName($moduleName, $targetItem)
                            );
                        }
                    }
                } else {
                    $query->select(array('id', 'name'));
                    $targets = $query->execute();
                }
                foreach($targets as $target){
                    self::$StackedRelatedEntities[$moduleName][$target['id']] = $target['name'];
                }
            }
        }

        foreach($recordIds as $recordId){
            if(!empty(self::$StackedRelatedEntities[$moduleName][$recordId])) {
                $result[$recordId] = array(
                    'id' => $recordId,
                    'name' => self::$StackedRelatedEntities[$moduleName][$recordId]
                );
            }
        }
        return $result;
    }

    public function getRelatedEntityName($moduleName, $recordId)
    {
        $relatedData = $this->getRelatedEntityData($moduleName, [$recordId]);
        $relatedItem = !empty($relatedData) ? array_shift($relatedData) : array();
        return !empty($relatedItem['name']) ? $relatedItem['name'] : '';
    }

    ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

    public function initChat(){
        if(!isset($this->last_comment_bean))
            $this->last_comment_bean = BeanFactory::getBean('PX_NeedToDoViewerComments');
    }

    public function addComment(PX_NeedToDoViewerComments $comment, $isNew = false)
    {
        if ($this->id && $comment->id && $comment->parent_id == $this->id) {
            if($isNew){
                $this->comment_count++;
            }
            $this->last_comment_bean = $comment;
            $this->save();
            return true;
        }
        return false;
    }

    public function deleteComment($comment_id)
    {
        //$this->initChat();

        if ($comment_id && $this->id) {
            $comment = BeanFactory::getBean("PX_NeedToDoViewerComments", $comment_id);
            if ($comment->parent_id == $this->id) {
                $comment->mark_deleted($comment_id);
                $this->comment_count--;
                $this->load_relationship('comments');
                $params = array('limit' => 1, 'orderby' => 'date_entered DESC');
                $linkResult = $this->comments->query($params);
                $last_comment_id = null;
                $linkResult = array_keys($linkResult['rows']);
                if (count($linkResult)) {
                    $last_comment_id = $linkResult[0];
                }
                $this->last_comment_bean = BeanFactory::getBean('PX_NeedToDoViewerComments', $last_comment_id);
                $this->save();
            }
        }
    }

    public function beforeSaveChat(){
        //$this->initChat();

        if(!empty($this->last_comment_bean))
            $this->last_comment = $this->last_comment_bean->toJson();
    }

    public function hasNewComments($needToDoEventId){
        global $current_user;

        $query = new SugarQuery();
        $query->from(BeanFactory::getBean('PX_NeedToDoViewerComments'));
        $commentUsersJoinA = $query->join('dialog_users', array('joinType' => 'INNER'))->joinName();
        $commentUsersJoinM2MA = $commentUsersJoinA . '_m2m';
        $query->where()->equals($commentUsersJoinM2MA.'.state', 'new');
        $query->where()->equals($commentUsersJoinM2MA.'.user_id', $current_user->id);
        $query->select(array('id'));
        $query->where()->equals('parent_type', 'PX_NeedToDoEvents')->equals('parent_id', $needToDoEventId);

        $query->select->selectReset()->setCountQuery();
        $queryResult = $query->execute();
        $recordCount = array_pop($queryResult);

        return $recordCount['record_count'] > 0;
    }

}