<?php
 if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');

require_once 'modules/PX_NeedToDoEvents/clients/base/api/PX_NeedToDoEventsFilterApi.php';

class PX_NeedToDoEventsApi extends PX_NeedToDoEventsFilterApi
{
    protected static $beanList = array();
    protected static $previewCheckResults = array();

    public function registerApiRest()
    {
        return array(
////            'record_px_timeline' => array(
            'need_to_do_events_context_record' => array(
                'reqType' => 'GET',
                'path' => array('pxntdv', '<module>','?', 'link', 'need-to-do-events'),
                'pathVars' => array('', 'module','record', '', ''),
                'method' => 'getNeedToDoListForRecord',
                'shortHelp' => '',
                'longHelp' => '',
            ),
////            'module_px_timeline' => array(
            'need_to_do_events_context_list' => array(
                'reqType' => 'GET',
                'path' => array('pxntdv', '<module>', 'PX_NeedToDoEvents'),
                'pathVars' => array('', 'module', ''),
                'method' => 'getNeedToDoListForModule',
                'shortHelp' => '',
                'longHelp' => '',
            ),
//            'home_px_timeline' => array(
            'need_to_do_events_on_home' => array(
                'reqType' => 'GET',
                'path' => array('pxntdv', 'Home', 'dashlet', 'PX_NeedToDoEvents'),
                'pathVars' => array('', '', '', ''),
                'method' => 'getNeedToDoListForHomePage',
                'shortHelp' => '',
                'longHelp' => '',
            ),
//            'record_px_timeline_count' => array(
            'need_to_do_events_context_record_count' => array(
                'reqType' => 'GET',
                'path' => array('pxntdv', 'count', '<module>','?', 'link', 'need-to-do-events'),
                'pathVars' => array('', '', 'module','record', '', ''),
                'method' => 'getNeedToDoEventsCountForRecord',
                'shortHelp' => '',
                'longHelp' => '',
            ),
//            'module_px_timeline_count' => array(
            'need_to_do_events_context_list_count' => array(
                'reqType' => 'GET',
                'path' => array('pxntdv', 'count', '<module>', 'PX_NeedToDoEvents'),
                'pathVars' => array('', '', 'module', ''),
                'method' => 'getNeedToDoEventsCountForModule',
                'shortHelp' => '',
                'longHelp' => '',
            ),
//            'home_px_timeline_count' => array(
            'need_to_do_events_on_home_count' => array(
                'reqType' => 'GET',
                'path' => array('pxntdv', 'count', 'Home', 'dashlet', 'PX_NeedToDoEvents'),
                'pathVars' => array('', '', '', '', ''),
                'method' => 'getNeedToDoEventsCountForHomePage',
                'shortHelp' => '',
                'longHelp' => '',
            ),
        );
    }

    public function getNeedToDoListForRecord(ServiceBase $api, array $args)
    {
        $params = $this->parseArguments($api, $args);
        $record = BeanFactory::retrieveBean($args['module'], $args['record']);

        if (empty($record)) {
            throw new SugarApiExceptionNotFound('Could not find parent record '.$args['record'].' in module '.$args['module']);
        }
        if (!$record->ACLAccess('view')) {
            throw new SugarApiExceptionNotAuthorized('No access to view records for module: '.$args['module']);
        }

        $query = $this->getNeedToDoEventQueryObject($api, $params, $record);
        return $this->formatResult($api, $args, $query, $record);
    }

    public function getNeedToDoListForModule(ServiceBase $api, array $args)
    {
        $params = $this->parseArguments($api, $args);
        $record = BeanFactory::getBean($args['module']);
        if (!$record->ACLAccess('view')) {
            throw new SugarApiExceptionNotAuthorized('No access to view records for module: '.$args['module']);
        }

        $query = $this->getNeedToDoEventQueryObject($api, $params, $record);
        return $this->formatResult($api, $args, $query, $record);
    }

    public function getNeedToDoListForHomePage(ServiceBase $api, array $args)
    {
        $params = $this->parseArguments($api, $args);
        $query = $this->getNeedToDoEventQueryObject($api, $params);
        return $this->formatResult($api, $args, $query);
    }

    public function getNeedToDoEventsCountForRecord(ServiceBase $api, array $args)
    {
        $record = BeanFactory::retrieveBean($args['module'], $args['record']);

        if (empty($record))
            throw new SugarApiExceptionNotFound('Could not find parent record '.$args['record'].' in module '.$args['module']);

        if (!$record->ACLAccess('view'))
            throw new SugarApiExceptionNotAuthorized('No access to view records for module: '.$args['module']);

        return $this->getNeedToDoEventsCount($api, $args, $record);
    }

    public function getNeedToDoEventsCountForModule(ServiceBase $api, array $args)
    {
        $record = BeanFactory::getBean($args['module']);
        if (!$record->ACLAccess('view'))
            throw new SugarApiExceptionNotAuthorized('No access to view records for module: '.$args['module']);

        return $this->getNeedToDoEventsCount($api, $args, $record);
    }

    public function getNeedToDoEventsCountForHomePage(ServiceBase $api, array $args)
    {
        return $this->getNeedToDoEventsCount($api, $args);
    }

    protected function getNeedToDoEventsCount(ServiceBase $api, array $args, $record = null){
        if(empty($args['filter']))
            $args['filter'] = array();
//        $args['filter'] = array_merge($args['filter'], $args['count_by_filter']);

        $params = $this->parseArguments($api, $args);
        $query = $this->getNeedToDoEventQueryObject($api, $params, $record);
        $query->select->selectReset()->setCountQuery();
//        $query->order_by = null;
        $query->limit = null;

        $orWhere = $query->where()->queryOr();
        $this->addControlDateLimitForDatetimeType($orWhere, $args['count_for_date']);
        $this->addControlDateLimitForDateType($orWhere, $args['count_for_date']);
        $query->orderByReset();

        $result = $query->execute();
        return reset($result);
    }

    protected function addControlDateLimitForDatetimeType(SugarQuery_Builder_Where $orWhere, $date){
        global $timedate, $current_user;



        $sugarDatetime = $timedate->fromDbDate($date);
        if(empty($date)){
            //TODO: Add Some conditions to query
            return;
        }

        $startIsoDatetime = gmdate(
            'Y-m-d\TH:i:s',
            gmmktime(0, 0, 0, $sugarDatetime->getMonth(), $sugarDatetime->getDay(), $sugarDatetime->getYear())
        );

        $endIsoDatetime = gmdate(
            'Y-m-d\TH:i:s',
            gmmktime(23, 59, 59, $sugarDatetime->getMonth(), $sugarDatetime->getDay(), $sugarDatetime->getYear())
        );

        $sfh = new SugarFieldHandler();
        $sugarField = $sfh->getSugarField('datetime');

        $startDatetimeValue = $sugarField->apiUnformatField($startIsoDatetime);
        $endDatetimeValue = $sugarField->apiUnformatField($endIsoDatetime);

        $andWhere = $orWhere->queryAnd();
        $andWhere
            ->equals('control_date_type', 'datetime')
            ->gte('control_date', $startDatetimeValue)
            ->lte('control_date', $endDatetimeValue);
    }

    protected function addControlDateLimitForDateType(SugarQuery_Builder_Where $orWhere, $date){
        global $timedate;

        $sugarDatetime = $timedate->fromDbDate($date);
        if(empty($date)){
            //TODO: Add Some conditions to query
            return;
        }

        $startSugarDate = $sugarDatetime->get_day_begin();

        $endSugarDate = $sugarDatetime->get_day_end_time();

        $startDate = $startSugarDate->format($timedate::DB_DATETIME_FORMAT);
        $endDate = $endSugarDate->format($timedate::DB_DATETIME_FORMAT);

        $andWhere = $orWhere->queryAnd();
        $andWhere->equals('control_date_type', 'date');
        $andWhere->gte('control_date', $startDate);
        $andWhere->lte('control_date', $endDate);
    }

//    public function parseArguments(ServiceBase $api, array $args)
    protected function parseArguments(ServiceBase $api, array $args, SugarBean $seed = null)
    {
        $params = parent::parseArguments($api, $args);
        if (isset($args['filter'])) {
            $params['filter'] = $args['filter'];
            $params['filter'] = $this->replaceNonDBFlexRelateFieldsInFilter($args['filter']);
        }
        if(isset($args['parentFilters'])){
            $params['parentFilters'] = $args['parentFilters'];
        }
        if(isset($args['dashlet_type'])){
            $params['dashlet_type'] = $args['dashlet_type'];
        }
        return $params;
    }

    protected function formatResult(ServiceBase $api, array $args, SugarQuery $query, SugarBean $bean = null)
    {
        global $locale;

        $response = array();
        $data = $query->execute();
        $seed = BeanFactory::newBean('PX_NeedToDoEvents');
        $limit = $query->limit - 1;
        $count = count($data);
        if ($count > $limit) {
            $nextOffset = $query->offset + $limit;
            array_pop($data);
        } else {
            $nextOffset = -1;
        }

        $options = array(
            'requestBean' => $bean,
        );

        foreach ($data as $row) {
            $seed->populateFromRow($row, true);
            $record = $this->formatBean($api, $args, $seed, $options);

//            if(!empty($record['context']) && is_string($record['context'])){
//                if($record['context'] != null)
//                    $record['context'] = array();
//                else
//                    $record['context'] = json_decode(htmlspecialchars_decode($record['context']), true);
//            }

            $record['parents_list'] = $seed->parents_list;

            $record['control_date_type'] = $row['control_date_type'];
            $record['control_date'] = $seed->formatControlDate($record['control_date_type'], $row['control_date']);

            if (!empty($record['object_type']) && !empty($record['object_id'])) {
                $previewCheckResult = $this->checkParentPreviewEnabled($api->user, $record['object_type'], $record['object_id']);
                $record['preview_enabled'] = $previewCheckResult['preview_enabled'];
                $record['preview_disabled_reason'] = $previewCheckResult['preview_disabled_reason'];
            }

            $record['created_by_name'] = $locale->formatName('Users', $row);
            if (!isset($record['created_by_name']) && isset($record['data']['created_by_name'])) {
                $record['created_by_name'] = $record['data']['created_by_name'];
            }
            if(isset($row['ntdv_notify_users_id'])){
                $record['ntdv_notify_users_id'] = $row['ntdv_notify_users_id'];
            }

            $response['records'][] = $record;
        }
        $response['next_offset'] = $nextOffset;
        $response['args'] = $args;
        return $response;
    }

//    protected function formatBean(ServiceBase $api, $args, SugarBean $bean, array $options = array()) {
    protected function formatBean(ServiceBase $api, array $args, SugarBean $bean, array $options = array()){
        $data = parent::formatBean($api, $args, $bean, $options);
//        $data['context'] = $bean->decodeContextField($data['context']);
//        $data['name_html'] = $bean->decodeNameHtmlField($data['name_html']);
//        $data['parent_attribute_name'] = $bean->getControlDateType($data['control_date']);
        return $data;
    }

    protected function checkParentPreviewEnabled($user, $module, $id)
    {
        $previewCheckKey = $module . '.' . $id;
        $previewCheckResult = array();
        if (array_key_exists($previewCheckKey, self::$previewCheckResults)) {
            $previewCheckResult = self::$previewCheckResults[$previewCheckKey];
        } else {
            $previewCheckBean = $this->getEmptyBean($module);
            if(!empty($previewCheckBean)) {
                $previewCheckBean->id = $id;
                //check if user has access - also checks if record is deleted
                $previewCheckResult['preview_enabled'] = $previewCheckBean->checkUserAccess($user);
                //currently only one error reason, but may be others in the future
                $previewCheckResult['preview_disabled_reason'] = $previewCheckResult['preview_enabled'] ? '' : 'LBL_PREVIEW_DISABLED_DELETED_OR_NO_ACCESS';
            }else{
                $previewCheckResult['preview_enabled'] = false;
                $previewCheckResult['preview_disabled_reason'] = 'LBL_PREVIEW_DISABLED_DELETED_OR_NO_ACCESS';
            }
        }
        self::$previewCheckResults[$previewCheckKey] = $previewCheckResult;
        return $previewCheckResult;
    }

    protected function getEmptyBean($module)
    {
        if (isset(self::$beanList[$module])) {
            $bean = self::$beanList[$module];
        } else {
            $bean = BeanFactory::getBean($module);
            if (!is_null($bean)) {
                self::$beanList[$module] = $bean;
            }
        }
        return $bean;
    }

    public function getNeedToDoEventQueryObject(ServiceBase $api, array $params, SugarBean $record = null)
    {
        if(!empty($params['dashlet_type']) &&  $params['dashlet_type'] == 'px-timeline-targetlist-dashlet'){
            return $this->getTargetListDependentNeedToDoQueryObject($api, $params, $record);
        }
         return $this->getContextDependentNeedToDoQueryObject($api, $params, $record);
    }

    //+++
    protected function getContextDependentNeedToDoQueryObject(ServiceBase $api, array $params, SugarBean $record = null){
        $seed = BeanFactory::getBean('PX_NeedToDoEvents');
        $query = new SugarQuery();
        $query->from($seed);

        // Always order by action date ASC.
        $query->orderBy('control_date', 'ASC');
        // +1 used to determine if we have more records to show.
        $query->limit($params['limit'] + 1)->offset($params['offset']);

        $columns = array('px_needtodoevents.*', 'users.picture');

        // Join with events names.
        $needToDoEventTypes = $query->join('event_type', array('joinType' => 'INNER'))->joinName();

        if (!empty($params['filter'])) {
            static::addFilters($params['filter'], $query->where(), $query);

            $this->addVisibilityFilters($query, $params);

            $this->addContextFilters($query, $params);

//            $targetListFilter = $this->getTargetListFilterDefinition($params['filter']);
//            if(!empty($targetListFilter)){
//                $this->applyTargetListFilter($targetListFilter, $query->where());
//            }
        }

        if(!empty($record) && !empty($record->id)){
            // If we have a relevant bean, we add our where condition.
//            $orWhere = $query->where()->queryOr();
//
//            //if current Record is Object Record of ntdv record
//            $subAndWnere1 = $orWhere->queryAnd();
//            $subAndWnere1->equals('px_needtodoevents.object_type', $record->module_name);
//            if($record->id){
//                $subAndWnere1->equals('px_needtodoevents.object_id', $record->id);
//            }

            //if current record is related to ntdv record
//            $subAndWnere2 = $orWhere->queryAnd();
            $parentContextFilterJta = 'px_ntdv_parents_context_filter';
            $parentContextFilterJoinTableSQL = "(SELECT event_id, parent_type, parent_id FROM px_ntdv_parents GROUP BY event_id, parent_type, parent_id)";
            $query->join[$parentContextFilterJta] = $query->joinTable($parentContextFilterJoinTableSQL, array('alias' => $parentContextFilterJta, 'joinType' => 'LEFT'));
            $query->join[$parentContextFilterJta]->on()
                ->equalsField($parentContextFilterJta . '.event_id', 'px_needtodoevents.id')
                ->equals($parentContextFilterJta . '.parent_type', $record->module_name)
                ->equals($parentContextFilterJta . '.parent_id', $record->id);
            $query->where()->isNotEmpty($parentContextFilterJta . '.event_id');

        } elseif(!empty($record)){
//            $query->where()->equals('px_needtodoevents.object_type', $record->module_name);

            $relatesQuery = new SugarQuery();
            $relatesQuery->from(BeanFactory::getBean('PX_NeedToDoRelates'));
            $relatesQuery->select('event_id');
            $relatesQuery->where()->equals('parent_type', $record->module_name);

            if(!empty($params['parentFilters'])){
                $parentQuery = new SugarQuery();
                $parentQuery->from($record);

                $apiRestRequest = $api->getRequest();
                $apiVersion = $apiRestRequest->getVersion();
                $request = array(
                    '__sugar_url' => 'v' . $apiVersion . '/' . $record->module_name,
                    'filters' => $params['parentFilters']
                );
                try {
                    require_once 'custom/PxRoutines/NeedToDoViewer/Include/api/NeedToDoRestService.php';
                    $restService = new NeedToDoRestService();
                    $apiClass = $restService->loadApiClassForTargetModule($request);
                    $apiClass::addFilters($params['parentFilters'], $parentQuery->where(), $parentQuery);
                }catch(Exception $e){
                    try {
                        static::addFilters($params['parentFilters'], $parentQuery->where(), $parentQuery);
                    }catch(Exception $e){
                        $GLOBALS['log']->error('Attention! Context filters can\'t be applied to need to do: ' . $e->getMessage());
                        $parentQuery = null;
                    }
                }

                if(!empty($parentQuery)) {
                    $parentQuery->select()->selectReset();
                    $parentQuery->select(array('id'));
                    $relatesQuery->where()->in('px_ntdv_parents.parent_id', $parentQuery);
                }
            }
            $query->where()->in('px_needtodoevents.id', $relatesQuery);
        }

//        $query->where()->equals('deleted', 0);
        $query->where()->equals("$needToDoEventTypes.status", PX_NeedToDoEventTypes::ENABLED_STATUS);
        $query->select($columns);
        $query->select()->addField("$needToDoEventTypes.icon", array('alias' => 'icon'));
        $query->select()->addField("$needToDoEventTypes.target_attribute_name", array('alias' => 'object_attribute_name'));
        $query->select()->addField("$needToDoEventTypes.name", array('alias' => 'event_type_name'));

//        $GLOBALS['log']->fatal('$query->compile()->getSQL(): ' . print_r($query->compile()->getSQL(), true));
//        $GLOBALS['log']->fatal('$query->compile()->getParameters(): ' . print_r($query->compile()->getParameters(), true));

        return $query;
    }

    protected function addContextFilters(SugarQuery $q, array $params){
        if(!empty($params['filter'])) {
            $contextEntityFilter = $this->getContextRelateFilter($params['filter']);
        }
        if(!empty($contextEntityFilter)) {
            $this->applyContextFilters($q, $contextEntityFilter);
        }
    }

    protected function addVisibilityFilters(SugarQuery $q, array $params){
        if(empty($params['filter']))
            return;

        $visibilityFilters = $this->getVisibilityFilterDefinition($params['filter']);
        if(
            empty($visibilityFilters) ||
            (
                in_array('current', $visibilityFilters) &&
                in_array('overdue', $visibilityFilters) &&
                in_array('future', $visibilityFilters)
            )
        ){
            return;
        }

        $orWhere = $q->where()->queryOr();
        foreach($visibilityFilters as $visibilityFilter) {
            if ($visibilityFilter == 'current') {
                $this->addCurrentVisibilityFilter($q, $orWhere);
            } elseif ($visibilityFilter == 'overdue') {
                $this->addOverdueVisibilityFilter($q, $orWhere);
            } elseif ($visibilityFilter == 'future') {
                $this->addFutureVisibilityFilter($q, $orWhere);
            }
        }

    }

    protected function addCurrentVisibilityFilter(SugarQuery $q, SugarQuery_Builder_Where $where){
        global $timedate;

//        $where = $q->where();
        $orWhere = $where->queryOr();

        $currentDateTimeFrom = $timedate->getNow();
        $currentDateTimeFrom->sub(new DateInterval('PT15M'));
        $currentDateTimeFormattedFrom = $currentDateTimeFrom->asDb();

        $currentDateTimeTo = $timedate->getNow();
        $currentDateTimeTo->add(new DateInterval('PT1H'));
        $currentDateTimeFormattedTo = $currentDateTimeTo->asDb();

        $andWhere = $orWhere->queryAnd();
        $andWhere
            ->equals('control_date_type', 'datetime')
            ->gte('control_date', $currentDateTimeFormattedFrom)
            ->lte('control_date', $currentDateTimeFormattedTo);



        $currentDate = $timedate->nowDbDate();
        $gmtTimezone = new DateTimeZone('GMT');

        $currentDateFrom = SugarDateTime::createFromFormat($timedate->get_db_date_time_format(),
            $timedate->merge_date_time($currentDate, '00:00:00'), $gmtTimezone)->format($timedate->get_db_date_time_format());

        $currentDateTo = SugarDateTime::createFromFormat($timedate->get_db_date_time_format(),
            $timedate->merge_date_time($currentDate, '23:59:59'), $gmtTimezone)->format($timedate->get_db_date_time_format());

        $andWhere = $orWhere->queryAnd();
        $andWhere->equals('control_date_type', 'date');
        $andWhere->gte('control_date', $currentDateFrom);
        $andWhere->lte('control_date', $currentDateTo);
    }

    protected function addOverdueVisibilityFilter(SugarQuery $q, SugarQuery_Builder_Where $where){
        global $timedate;

//        $where = $q->where();
        $orWhere = $where->queryOr();

        $overdueDateTime = $timedate->getNow();
        $overdueDateTime->sub(new DateInterval('PT15M'));
        $overdueDateTimeFormattedTo = $overdueDateTime->asDb();
        $andWhere = $orWhere->queryAnd();
        $andWhere->equals('control_date_type', 'datetime')->lt('control_date', $overdueDateTimeFormattedTo);



        $currentDate = $timedate->nowDbDate();
        $gmtTimezone = new DateTimeZone('GMT');
        $overdueDateTimeTo = $date = SugarDateTime::createFromFormat($timedate->get_db_date_time_format(),
            $timedate->merge_date_time($currentDate, '00:00:00'), $gmtTimezone)->format($timedate->get_db_date_time_format());
        $andWhere = $orWhere->queryAnd();
        $andWhere->equals('control_date_type', 'date');
        $andWhere->lte('control_date', $overdueDateTimeTo);
    }

    protected function addFutureVisibilityFilter(SugarQuery $q, SugarQuery_Builder_Where $where){
        global $timedate;

//        $where = $q->where();
        $orWhere = $where->queryOr();

        $futureDateTimeFrom = $timedate->getNow();
        $futureDateTimeFrom->add(new DateInterval('PT1H'));
        $futureDateTimeFormattedFrom = $futureDateTimeFrom->asDb();

        $andWhere = $orWhere->queryAnd();
        $andWhere
            ->equals('control_date_type', 'datetime')
            ->gte('control_date', $futureDateTimeFormattedFrom);



        $futureDateTimeFrom = $timedate->getNow();
        $futureDateTimeFrom->add(new DateInterval('P1D'));
        $futureDateTimeFormattedFrom = $futureDateTimeFrom->asDbDate();

        $currentDate = $timedate->nowDbDate();
        $gmtTimezone = new DateTimeZone('GMT');
        $futureDateFrom = $date = SugarDateTime::createFromFormat($timedate->get_db_date_time_format(),
            $timedate->merge_date_time($futureDateTimeFormattedFrom, '00:00:00'), $gmtTimezone)->format($timedate->get_db_date_time_format());;
        $andWhere = $orWhere->queryAnd();
        $andWhere
            ->equals('control_date_type', 'date')
            ->gte('control_date', $futureDateFrom);
    }

    protected function getVisibilityFilterDefinition($filterList){
        foreach($filterList as $key => $filter){
            foreach($filter as $fieldName => $filterDefinition){
                if(!empty($fieldName) && $fieldName == 'event_status'){
                    if(!empty($filterDefinition['$in'])){
                        return $filterDefinition['$in'];
                    }
                    return $filterDefinition;

                }
            }
        }
        return null;
    }

    protected function getTargetListDependentNeedToDoQueryObject(ServiceBase $api, array $params, SugarBean $record = null)
    {
        $seed = BeanFactory::getBean('PX_Timeline');
        $query = new SugarQuery();
        $query->from($seed);
        $query->orderBy('action_date', 'DESC');
        $query->limit($params['limit'] + 1)->offset($params['offset']);
        $columns = array('px_timeline.*', 'users.picture');
        $flowchartEvents = $query->join('event', array('joinType' => 'LEFT'))->joinName();

        if (!empty($params['filter'])) {
            static::addFilters($params['filter'], $query->where(), $query);

            $targetListFilter = $this->getTargetListFilterDefinition($params['filter']);
            if(!empty($targetListFilter)){
                $this->applyTargetListFilter($targetListFilter, $query->where());
            }
        }

        if(!empty($record) && !empty($record->id)){
            $this->applyProspectListFilters($query->where()->queryOr(), '$in', $record->id);
        } elseif(!empty($record)){
            if(!empty($params['parentFilters']) && $record instanceof ProspectList){
                if(!empty($params['parentFilters'])){
                    $this->applyContextProspectListFilters($query->where()->queryOr(), $params['parentFilters']);
                }
            }
        }

        $query->where()->equals('deleted', 0);
        $query->select($columns);
        $query->select()->addField("$flowchartEvents.icon", array('alias' => 'icon'));
        $query->select()->addField("$flowchartEvents.name", array('alias' => 'event_type_name'));

        return $query;
    }

    protected function searchTargetFilters($filterList){
        //$walk = function($array) use ( &$walk ) {
            $searchedKeys = array('object_id', 'object_type', 'filter_object_type');
            if( !is_array( $filterList)) {
                return false;
            }
            foreach ($filterList as $k => $v) {
                if($k && in_array($k, $searchedKeys)){
                    return true;
                }
                $data = $this->searchTargetFilters($v);
                if($data != false){
                    return $data;
                }
            }
            return false;
    }

    protected function getParentTypeFromFilter($filterList, $isFound = false){
            $result = array();
            if( !is_array( $filterList)) {
                    return $isFound ? array($filterList) : array();
            }
            foreach ($filterList as $k => $v) {
                if($k && $k == 'object_type'){
                    $r = $this->getParentTypeFromFilter($v, true);
                }else{
                    $r = $this->getParentTypeFromFilter($v, $isFound);
                }
                if(!empty($r)){
                    $result = array_merge($result, $r);
                }
            }
            return $result;
    }
}
