<?php

require_once 'clients/base/api/ModuleApi.php';

class PX_NeedToDoEventsModuleApi extends ModuleApi {

    public function registerApiRest() {
        return array(
            'retrieve' => array(
                'reqType' => 'GET',
                'path' => array('PX_NeedToDoEvents','?'),
                'pathVars' => array('module','record'),
                'method' => 'retrieveRecord',
                'shortHelp' => 'Returns a single record',
                'longHelp' => 'include/api/help/module_record_get_help.html',
            ),
        );
    }

    protected function formatBean(ServiceBase $api, array $args, SugarBean $bean, array $options = array())
    {

        $data = parent::formatBean($api, $args, $bean, $options);

        $data['notify_users_list'] = $this->getNotifyUsersNames($data);
        $data['parents_list'] = $bean->populateParentsList($data['id'], $data['event_type_id']);

        return $data;
    }

    public function getNotifyUsersNames($record){
        $notifyUsers = $this->getNotifyUserIds($record);
        $userNames = array();
        if(empty($notifyUsers)){
            return '';
        }
        $notifyUserIds = array();
        foreach($notifyUsers as $notifyUser){
            $userId = $notifyUser['id'];
            $notifyUserIds[$userId] = $userId;
        }

        if(!empty($notifyUserIds)){
            $query = new SugarQuery();
            $query->from(BeanFactory::getBean('Users'));
            $query->select(array('id', 'first_name', 'last_name'));
            $query->where()->in('id', $notifyUserIds);
            $queryResult = $query->execute();

            global $locale;
            if (!empty($queryResult)) {
                foreach($queryResult as $userName) {
                    $userNames[$userName['id']] = $locale->formatName('Users', $userName);
                    unset($notifyUserIds[$userName['id']]);
                }
            }

            foreach($notifyUserIds as $notifyUserId){
                $userNames[$notifyUserId] = '';
            }
        }

        foreach($notifyUsers as $key => $notifyUser){
            if(isset($userNames[$notifyUser['id']])){
                $notifyUsers[$key]['name'] = $userNames[$notifyUser['id']];
            }
        }
        return $this->jsonEncodeFromArray($notifyUsers);
    }

    private function getNotifyUserIds($record){
        $notifyUserIds = array();
        $db = \DBManagerFactory::getInstance();
        $sql = "
            SELECT user_id, notified
            FROM px_ntdv_notify_users
            WHERE event_id = '" . $record['id'] . "'
        ";
        $rs = $db->query($sql);
        while($row = $db->fetchByAssoc($rs)){
            $notifyUserIds[] = array(
                'id' => $row['user_id'],
                'notified' => $row['notified']
            );
        }

        return $notifyUserIds;
    }

    private function jsonEncodeFromArray($field){
        if(empty($field) ) {
            return '';
        }elseif(is_string($field)) {
            return $field;
        }

        return json_encode($field);
    }

}
