({
    events: {
        'click .preview-btn:not(.disabled)': 'previewRecord',
    },

    className: "ntdv-row-item",
    plugins: ['RelativeTime', 'FileDragoff'],
    cacheNamePrefix: "user:avatars:",
    cacheNameExpire: ":expiry",
    expiryTime: 36000000,   //1 hour in milliseconds

    _attachImageSelector: 'img',

    blacklistModules: [
        'Quotas'
    ],

    previewModel: null,

    dateLineDelimiter: false,
    datetimeDelimiter: '',
    displayContext: {
        object_record: true,
        subject_record: true,
        context: true,
    },
    nopreview: '',

    isCollapsed: false,

    href: '',

    commentBlock: null,

    commentsNestingLevel: 2,


    initialize: function(options) {
        this.defineCustomHbsHelpers();

        this.opts = {params: {}};
        this.readonly = !!options.readonly;
        this.nopreview = !!options.nopreview;

        this.dateLineDelimiter = options.dateLineDelimiter;

        Handlebars.registerHelper("ntdvBuildUrl", function(module, id){
            var route = app.router.buildRoute(module, id, '');

            return '#' + route;
        });
        Handlebars.registerHelper("ntdvBuildRecordsList", function(records){
            var recordUrls = [];

            _.each(records, function(record){
                var moduleName = record.moduleName || '',
                    recordId = record.recordId || '',
                    recordName = record.recordName || '';
                if(!_.isEmpty(moduleName) || !_.isEmpty(recordId) || !_.isEmpty(recordName)) {
                    recordUrls.push(
                        '<a href="#' + app.router.buildRoute(moduleName, recordId, '') + '">' + recordName +'</a>'
                    );
                }
            });

            return recordUrls.join(', ');
        });

        app.view.View.prototype.initialize.call(this, options);

        this.events['click .comment-btn[data-idAction="' + this.cid + '"]:not(.disabled)'] = 'toggleReplyBar';
        this.events['click .show-comments-btn[data-idAction="' + this.cid + '"]:not(.disabled)'] = 'showComments';
        this.events['click .context-btn[data-idAction="' + this.cid + '"]:not(.disabled)'] = 'contextToggled';

        /* COMMENT-NEW */
        this.commentBlock = app.view.createView({
            context: this.context,
            name: "need-to-do-event-comments",
            type: "need-to-do-event-comments",
            module: 'PX_NeedToDoViewerComments',
            layout: this,
            model: this.model,
            isReply: false,
            commentsNestingLevel: this.commentsNestingLevel,
            needToDoEventLayout: this.layout
        });
        this.setExtendedContextDescription();
        this.preview = this.getPreviewData();
        this.tpl = "TPL_ACTIVITY_LINK";


        // if(this.dateLineDelimiter){
        //     this._setDatetimeDelimiter();
        // }


    },

    contextToggled: function(e){
        var el = $(e.currentTarget),
            targetId = $(el).data('target'),
            isExpanded = $(el).data('pressed'),
            contextToggleEl = this.$el.find('.ntdv-context-slide-container'),
            contextEl = this.$el.find('.ntdv-context-container'),
            ntdvPanel = this.$el.find('.ntdv-panel'),
            contextElHeight = contextEl[0].offsetHeight,

            needToDoContainer = ntdvPanel.closest('.ntdv-container'),
            needToDoRowItemContainer = needToDoContainer.closest('.ntdv-row-item'),
            needToDoContainerName = needToDoContainer.attr('name'),
            needToDoContainerNameTopOffset = this.layout.getTargetNeedToDoContainerPosition(needToDoRowItemContainer, needToDoContainer.closest('.ntdv-group'));

        if(!contextToggleEl.length) {
            return;
        }

        if(!isExpanded){
            this.$el.find('.ntdv-context-container .ntdv-group-outliner').show();
        }

        this.toggleActionButton(el, !isExpanded);

        contextToggleEl
            .animate({
                    height: "toggle"
                }, 100,
                _.bind(function() {
                    if(isExpanded) {
                        this.$el.find('.ntdv-context-container .ntdv-group-outliner').hide();
                    }

                    if(isExpanded) {
                        var stickyItemEl = this.layout.getStickyItemEl();
                        if (stickyItemEl && stickyItemEl.data('name') == needToDoContainerName) {
                            this.layout.updateScrollPositionOnToggleNeedToDoDetails(needToDoContainerNameTopOffset);
                        }
                        if(!this.layout.needToDoPanelHasAdditionalContainers(needToDoContainer.closest('.ntdv-container')))
                            this.layout.clearUpStickyItem();
                    }

                }, this));

    },

    fixScrollBarsForIframe: function($contextContainer){
        $contextContainer.find('iframe').each(function(){
            var iframe = this;
            iframe.contentDocument.location.reload();
        });
    },

    isElementInViewport: function(el, parent) {
        var top = el.offsetTop,
            height = el.offsetHeight;

        var parentTop = parent.offsetTop + parent.scrollTop,
            parentHeight = parent.offsetHeight;

        while(el.offsetParent) {
            el = el.offsetParent;
            top += (el.offsetTop);
        }

        while(parent.offsetParent) {
            parent = parent.offsetParent;
            parentTop += (parent.offsetTop);
        }

        var isInViewPort = {
            top : (top) >= (parentTop) && (top) <= (parentTop + parentHeight),
            bottom: (top + height) <= (parentTop + parentHeight)
        };

        return isInViewPort;
    },

    setExtendedContextDescription: function(){

        var contextModuleName = this.context.parent.get('module'),
            layoutLayoutType = this.context.parent.get('layout'),
            layoutName = this.layout.name;

        if (layoutName == 'need-to-do-events-dashlet' || layoutName == 'px-timeline-targetlist-dashlet') {
            layoutLayoutType = layoutName;
        }

        switch (layoutLayoutType) {
            case "records":
                this.displayContext.object_record = true;
                this.displayContext.subject_record = true;
                this.displayContext.context = false;
                break;
            case "record":
                this.displayContext.object_record = false;
                this.displayContext.subject_record = true;
                this.displayContext.context = false;
                break;
            // case "px-timeline":
            case "need-to-do-events-dashlet":
                this.displayContext.object_record = true;
                this.displayContext.subject_record = true;
                this.displayContext.context = true;
                break;
            // case "px-timeline-targetlist-dashlet":
            //     this.displayContext.object_record = true;
            //     this.displayContext.subject_record = true;
            //     this.displayContext.context = true;
            //     break;
        }
    },

    /**
     * Handler for previewing a record listed on the activity stream.
     * @param  {Event} event
     */
    previewRecord: function(event) {
        var context = this.model.get('context');
        var collection = this.layout.previewCollection || this.context.get("collection");

        if(!this.isContextEmpty(context)){
            this.decorateRow();
            app.events.trigger("preview:render", this.model, collection, true, this.cid);
        }else{
            var module = this.model.get('object_type'),
                id = this.model.get('object_id');

            if (module && id) {
                if(_.isEmpty(this.previewModel)) {
                    this.previewModel = app.data.createBean(module);
                    this.previewModel.set("id", id);
                    this.previewModel.set('px_timeline_id', this.model.get('id'));
                }
                this.decorateRow();
                app.events.trigger("preview:render", this.previewModel, collection, true, this.cid);
            }
        }

        event.preventDefault();
    },

    /**
     * Handles highlighting of current activity item and active state of preview button.
     */
    decorateRow: function() {
        this.layout.decorateRow(this.model);

    },

    isGroupedCollapsed: function(){
        var key = this.layout.getStatisticKey(this.model.get('action_date'));
        return this.layout.needToDoStatistics[key].view.isCollapsed;
    },

    _renderHtml: function(model) {

        // if(this.displayContext.context) {
        //     this.displayContext.context = !this.isContextEmpty(this.model.get('context'));
        // }

        // if(this.isGroupedCollapsed()){
        //     this.hide();
        // }

        this.groupParentsList();
        this.processAvatars();
        this.buildHref();
        // this._setActionTime();
        this.setControlDatetime();
        this.setObjectTypeLabel();
        // this.prepareNameHtml();
        app.view.View.prototype._renderHtml.call(this);

        /* COMMENT-NEW */
        this.$el.find('.ntdv-comment-container').append(this.commentBlock.$el);
        this.commentBlock.render();

        // this.renderContextField();

        // this._addBrokenImageHandler();
    },

    groupParentsList: function(){
        var parentsList = this.model.get('parents_list'),
            groupedParentList = {};
        _.each(parentsList, function(parentItem){
            if(!_.isEmpty(parentItem['recordName'])) {
                var moduleName = parentItem['moduleName'],
                    linkName = parentItem['linkName'],
                    linkLabel = parentItem['linkLabel'];
                groupedParentList[moduleName] = groupedParentList[moduleName] || {};
                groupedParentList[moduleName][linkName] = groupedParentList[moduleName][linkName] || {
                    linkLabel: linkLabel,
                    records: []
                };
                groupedParentList[moduleName][linkName]['records'].push(parentItem);
            }
        });

        this.model.set('parents_list', groupedParentList);
    },

    prepareNameHtml: function(){
        var html = this.model.get('name_html'),
            isEmpty = true;
        if(!_.isEmpty(html)){
            isEmpty = $("<p>").html(html).text().trim().length;
            if(!isEmpty){
                this.model.set('name_html', '');
            }
        }
    },

    renderContextField: function(){
        var fieldDefs = _.extend(this.model.fields.context, {type: 'px-context'});
        var obj = {
            meta: {
                view: "detail"
            },
            def: fieldDefs,
            model: this.model,
            context: app.controller.context,
            viewName: "detail",
            view: this
        };
        var field = app.view.createField(obj);
        field.action = 'detail';


        var $fieldPlaceholder,
            $contextContainer = this.$el.find('.context .px-context-container');

        $fieldPlaceholder = $(field.getPlaceholder().string);
        $fieldPlaceholder.appendTo($contextContainer);

        this._renderField(field, $fieldPlaceholder);

        return field;
    },

    /**
     * Add a listener for when activity has a broken image for attach type posts
     * Remove the broken image and remove link to broken image
     *
     * @private
     */
    _addBrokenImageHandler: function() {
        this.$(this._attachImageSelector).on('error', _.bind(function(event) {
            var $brokenImg = $(event.currentTarget);
            $brokenImg.attr('src', 'custom/PxRoutines/NeedToDoViewer/Images/broken_image_icon.png');
        }, this));
    },

    /**
     * Sets property on activity to show date created as a relative time or as date time.
     *
     * @private
     */
    _setDatetimeDelimiter: function() {
        var action_date = this.model.get('action_date');
        this.datetimeDelimiter = app.date(action_date).format("dddd, MMMM Do, YYYY");
    },

    setControlDatetime: function() {
        var control_time = app.date(this.model.get('control_date')).format(app.date.convertFormat(this.getUserTimeFormat()));
        this.model.set('control_time', control_time);
    },

    setObjectTypeLabel: function() {
        var objectTypeLabel = '',
            moduleSingularList = app.lang.getAppListStrings('moduleListSingular');
        moduleSingularList = moduleSingularList || app.lang.getAppListStrings('moduleList');
        if(!_.isEmpty(moduleSingularList)){
            objectTypeLabel = moduleSingularList[this.model.get('object_type')];
        }
        objectTypeLabel = objectTypeLabel || this.model.get('object_type');

        this.model.set('object_type_label', objectTypeLabel);
    },

    getUserTimeFormat: function() {
        return app.user.getPreference('timepref');
    },

    /**
     * Sets the profile picture for activities based on the created by user.
     */
    processAvatars: function () {
        if (!this.model.get('icon_url')) {
            var iconUrl = this.buildIconUrl(this.model.get('event_type_id'));
            this.model.set('icon_url', iconUrl);
        }

        var comments = this.model.get('comments');
        if (comments) {
            comments.each(function (comment) {
                var commentPictureUrl = this.getAvatarUrlForUser(comment, 'comment');
                comment.set('picture_url', commentPictureUrl);
            }, this);
        }
    },

    getAvatarUrlForUser: function (model, activityType){
        var createdBy = model.get('created_by'),
            hasPicture = this.checkUserHasPicture(model, activityType);

        return hasPicture ? this.buildAvatarUrl(createdBy) : '';
    },

    checkUserHasPicture: function (model) {
        var createdBy = model.get('created_by'),
            hasPicture;

        // If processing the current user's avatar, no need to fetch
        if (createdBy === app.user.get('id')) {
            hasPicture = !_.isEmpty(app.user.get('picture'));
        } else {
            // Check cache
            hasPicture = this.getUserPictureStatus(createdBy);
        }

        // If not current user or cached, call api to check if user has a picture
        if (_.isUndefined(hasPicture)) {
            this.fetchUserPicture(model);
            hasPicture = false; // Use placeholder until api call finishes
        }

        return hasPicture;
    },

    fetchUserPicture: function(model) {
        var self = this,
            createdBy = model.get('created_by'),
            user = app.data.createBean('Users', {id: createdBy});

        user.fetch({
            fields: ["picture"],
            success: function () {
                var pictureUrl = self.buildAvatarUrl(createdBy),
                    hasPicture = !_.isEmpty(user.get('picture'));

                self.setUserPictureStatus(createdBy, hasPicture);

                // If picture exists, replace the activity image with the user's profile picture
                if (hasPicture) {
                    self.$('#ntdv-comment-author-avatar-' + model.get('id')).html("<img src='" + pictureUrl + "' alt='" + model.get('created_by_name') + "'>");
                }
            },
            error: function () {
                // Problem retrieving picture, use placeholder
                self.setUserPictureStatus(createdBy, false);
            }
        });
    },

    setUserPictureStatus: function(userId, hasPicture) {
        app.cache.set(this.cacheNamePrefix + userId, hasPicture);
        app.cache.set(this.cacheNamePrefix + userId + this.cacheNameExpire, $.now() + this.expiryTime);
    },

    getUserPictureStatus: function(userId) {
        var hasPicture = app.cache.get(this.cacheNamePrefix + userId),
            cachedTTL = app.cache.get(this.cacheNamePrefix + userId + this.cacheNameExpire);

        return (cachedTTL < $.now()) ? undefined : hasPicture;
    },

    buildIconUrl: function(eventId) {
        return app.api.buildFileURL({
            module: 'PX_NeedToDoEventTypes',
            id: eventId,
            field: 'icon'
        });
    },

    /**
     * Build the file url for the given user's avatar
     *
     * @param userId
     * @return {string} The avatar url
     * @private
     */
    buildAvatarUrl: function(userId) {
        return app.api.buildFileURL({
            module: 'Users',
            id: userId,
            field: 'picture'
        });
    },

    /**
     * Determines the status and label for the preview button.
     *
     * @return {Object}
     * @return {Boolean} return.enabled Whether the preview is enabled.
     * @return {String} return.label The label to display in the preview button
     *   tooltip.
     */
    getPreviewData: function () {
        var parentModel,
            preview = {
                enabled: true,
                label: 'LBL_PREVIEW'
            },
            ntdvContext = this.model.get('context'),
            ntdvContextIsEmpty = this.isContextEmpty(ntdvContext),
            isBwcEnabled,
            module = this.model.get('object_type');

        if (module && ntdvContextIsEmpty) {
            isBwcEnabled = true;
            var moduleMetadata = app.metadata.getModule(module);
            if (moduleMetadata && _.has(moduleMetadata, 'isBwcEnabled')) {
                isBwcEnabled = moduleMetadata.isBwcEnabled;
            }
        } else if(ntdvContextIsEmpty) {
            isBwcEnabled = false;
        }else {
            isBwcEnabled = false;
        }

        if (isBwcEnabled) {
            preview.enabled = false;
            preview.label = 'LBL_NTDV_PREVIEW_BWC_TOOLTIP';
        } else if (ntdvContextIsEmpty &&
            (_.isEmpty(this.model.get('object_id')) || _.isEmpty(this.model.get('object_type')))) {
            preview.enabled = false;
            preview.label = 'LBL_NTDV_PREVIEW_DISABLED_NO_RECORD';
        } else if (!app.acl.hasAccess("view", this.model.get('object_type'))) {
            preview.enabled = false;
            preview.label = 'LBL_NTDV_PREVIEW_DISABLED_NO_ACCESS';
        }
        else {
            parentModel = this._getParentModel('record', this.context);
            if (parentModel && parentModel.module == this.model.get('object_type') && parentModel.id === this.model.get('object_id')) {
                preview.enabled = false;
                preview.label = 'LBL_NTDV_PREVIEW_DISABLED_SAME_RECORD';
            }
        }

        return preview;
    },

    isContextEmpty: function(ntdvContext){
        if(_.isEmpty(ntdvContext) || ntdvContext == 'null')
            return true;

        return this.isContextElementEmpty(ntdvContext);
    },

    isContextElementEmpty: function(ntdvContext){
        var isEmpty = true;
        _.each(ntdvContext, function(el){
            el = el || {};
            el['text'] = el['text'] || '';
            if(_.isNumber(el['text']) && !_.isNaN(el['text'])){
                isEmpty = false;
                return;
            }
            else if(!_.isEmpty(el['text']) && el['text'] !== ''){
                isEmpty = false;
                return;
            }else if(!_.isEmpty(el['list'])){
                if(!this.isContextElementEmpty(el['list'])){
                    isEmpty = false;
                    return;
                }
            }
        }, this);

        return isEmpty;
    },


    /**
     * Traverse up the context hierarchy and look for given layout, retrieve the model from the layout's context
     *
     * @param layoutName to look for up the context hierarchy
     * @param context start of context hierarchy
     * @return {Mixed}
     * @private
     */
    _getParentModel: function(layoutName, context) {
        if (context) {
            if (context.get('layout') === layoutName) {
                return context.get('model');
            } else {
                return this._getParentModel(layoutName, context.parent);
            }
        } else {
            return null;
        }
    },

    /**
     * Data change event.
     */
    bindDataChange: function () {
        app.view.View.prototype.bindDataChange.call(this);
    },

    unbindData: function() {
        app.view.View.prototype.unbindData.call(this);
    },

    _dispose: function() {
        $(window).off('resize.' + this.cid);
        this.$(this._attachImageSelector).off('error');
        app.view.View.prototype._dispose.call(this);
        //this.commentsCollection = null;
        this.opts = null;
    },

    refreshAmount: function(displayedAmount, totalAmount){
        var $el = this.$el.find('.ntdv-count'),
            label = app.lang.get('LBL_COUNT_DISPLAYED_ALL_RECORDS', 'PX_NeedToDoEvents');
        if(!_.isEmpty(totalAmount) && displayedAmount < totalAmount){
            label = app.lang.get('LBL_COUNT_DISPLAYED_PART_OF_RECORDS', 'PX_NeedToDoEvents');
            label = label.replace("total_amount", "" + totalAmount + "");
        }
        label = label.replace("displayed_amount", "" + displayedAmount + "");

        if(!_.isEmpty($el)){
            $el.html(label);
        }
    },

    buildHref: function() {
        var module = 'PX_NeedToDoEvents';
        this.href = '#' + app.router.buildRoute(module, this.model.get('id'));
    },

    /* COMMENTS */

    toggleReplyBar: function(e) {
        this.commentBlock.toggleReplyBar(e.currentTarget);
    },

    showComments: function(e){
        var el = e.currentTarget,
            commentListEl = this.$el.find('.comments-list').first(),
            ntdvPanel = this.$el.find('.ntdv-panel'),
            needToDoContainer = ntdvPanel.closest('.ntdv-container'),
            needToDoRowItemContainer = needToDoContainer.closest('.ntdv-row-item'),
            needToDoContainerName = needToDoContainer.attr('name'),
            needToDoContainerNameTopOffset = this.layout.getTargetNeedToDoContainerPosition(needToDoRowItemContainer, needToDoContainer.closest('.ntdv-group')),
            stickyItemEl = this.layout.getStickyItemEl();

        if($(el).data('pressed')) {
            if(!_.isEmpty(needToDoContainerName) && stickyItemEl && stickyItemEl.attr('name') == needToDoContainerName) {
                this.$el.find('.ntdv-comment-container').removeClass('expanded');
                this.$el.find('.ntdv-comment-container .ntdv-group-outliner').hide();
                var commentsEl = this.$el.find('.ntdv-comment-container'),
                    commentsElHeight = commentsEl[0].offsetHeight;
                this.commentBlock.hideNeedToDoEventComments(commentListEl);
                this.toggleActionButton(el, false);

                this.layout.updateScrollPositionOnToggleNeedToDoDetails(needToDoContainerNameTopOffset);
            }else{
                this.commentBlock.slideHideNeedToDoEventComments(commentListEl, _.bind(function(){
                    _.each(this.$el.find('.show-comments-btn'), function(el){
                        this.toggleActionButton(el, false);
                    }, this);
                    this.$el.find('.ntdv-comment-container').removeClass('expanded');
                    this.$el.find('.ntdv-comment-container .ntdv-group-outliner').hide();
                }, this));
            }

            if(!this.layout.needToDoPanelHasAdditionalContainers(needToDoContainer)) {
                this.layout.clearUpStickyItem();
            }
        }else {
            this.$el.find('.ntdv-comment-container').addClass('expanded');
            this.$el.find('.ntdv-comment-container .ntdv-group-outliner').show();
            this.commentBlock.slideShowNeedToDoEventComments(commentListEl, _.bind(function(){
            }, this));
            _.each(this.$el.find('.show-comments-btn'), function(el){
                this.toggleActionButton(el, true);
            }, this);
        }
    },

    toggleActionButton: function(el, activated) {
        if(activated === undefined) {
            activated = $(el).hasClass('active');
        }

        $(el).toggleClass('active', activated).attr('aria-pressed', activated).data('pressed', activated);
    },

    addComment: function (event) {
        var self = this,
            parentId = this.model.id,
            payload = {
                // timeline_id: parentId,
                data: {}
            },
            bean;

        payload.data = this.getComment();

        if (payload.data.value && (payload.data.value.length > 0)) {
            bean = app.data.createRelatedBean(this.model, null, 'comments');

            bean.save(payload, {
                relate: true,
                success: _.bind(self.addCommentCallback, self)
            });
        }
    },

    /**
     * Retrieve comment entered inside content editable and translate any tags into text format
     * so that it can be saved in the database as JSON string.
     *
     * @return {string}
     */
    getComment: function() {
        return this.unformatTags(this.$('div.reply'));
    },

    addCommentCallback: function (model) {
        var template, data;

        this.$('div.reply').empty().trigger('change');
        this.commentsCollection.add(model);


        var commentElement = this.$('.comment-btn:first');
        this.toggleReplyBar();

        template = app.template.getView('px-timeline-row.comment', 'PX_NeedToDoEvents' /*this.module*/);

        data = model.get('data');
        data.value = this.formatTags(data.value);
        data.value = this.formatLinks(data.value);

        this.processAvatars();
        this.$('.ntdv-comment-container').find('.comment:first').before(template(model.attributes));
    },

    formatLinks: function(post) {
        var formattedPost = '';

        if (post && (post.length > 0)) {
            formattedPost = post.replace(this.urlRegExp, function(url) {
                var href = url;
                if ((url.indexOf('http://') !== 0) && (url.indexOf('https://') !== 0)) {
                    href = 'http://' + url;
                }
                return '<a href="' + href + '" target="_blank">' + url + '</a>';
            });
        }

        return formattedPost;
    },

    checkPlaceholder: function(e) {
        var el = e.currentTarget;
        if (el.textContent) {
            el.setAttribute('data-hide-placeholder', 'true');
            this.toggleSubmitButton(true);
        } else {
            el.removeAttribute('data-hide-placeholder');
            this.toggleSubmitButton(false);
        }
    },

    toggleSubmitButton: function(enable) {
        if (!enable) {
            this.disableSubmitButton();
        } else {
            this.enableSubmitButton();
        }
    },

    /**
     * Enable Submit button
     */
    enableSubmitButton: function() {
        this.$('.reply-btn')
            .removeClass('disabled')
            .attr('aria-disabled', false)
            .attr('tabindex', 0);
    },

    /**
     * Disable Submit button
     */
    disableSubmitButton: function() {
        this.$('.reply-btn')
            .addClass('disabled')
            .attr('aria-disabled', true)
            .attr('tabindex', -1);
    },

    fetchComments: function() {
        var self = this;
        this.commentsCollection.fetch({
            //Don't show alerts for this request
            showAlerts: false,
            relate: true,
            success: function(collection) {
                self.remaining_comments = 0;
                self.render();
            }
        });
    },

    defineCustomHbsHelpers: function(){
        Handlebars.registerHelper('renderNtdvSubTpl', function(key, data, options) {
            var frame, template;

            template = app.template.getView(key, 'PX_Timeline');
            frame = _.extend(Handlebars.createFrame(options.data || {}), options.hash);

            return template ? template(data, {data: frame}) : '';
        });
    }
})
