<?php

require_once 'clients/base/api/FilterApi.php';

class PX_NeedToDoViewerCommentsFilterApi extends FilterApi{

	public function registerApiRest()
	{
		return array(
			'filterModuleGet' => array(
				'reqType' => 'GET',
				'path' => array('PX_NeedToDoViewerComments', 'filter'),
				'pathVars' => array('module', ''),
				'method' => 'filterList',
				'jsonParams' => array('filter')
			),
			'filterModuleAll' => array(
				'reqType' => 'GET',
				'path' => array('PX_NeedToDoViewerComments'),
				'pathVars' => array('module'),
				'method' => 'filterList',
				'jsonParams' => array('filter')
			),
			'filterModuleAllCount' => array(
				'reqType' => 'GET',
				'path' => array('PX_NeedToDoViewerComments', 'count'),
				'pathVars' => array('module', ''),
				'jsonParams' => array('filter'),
				'method' => 'getFilterListCount',
			),
			'filterModulePost' => array(
				'reqType' => 'POST',
				'path' => array('PX_NeedToDoViewerComments', 'filter'),
				'pathVars' => array('module', ''),
				'method' => 'filterList',
			),
			'filterModulePostCount' => array(
				'reqType' => 'POST',
				'path' => array('PX_NeedToDoViewerComments', 'filter', 'count'),
				'pathVars' => array('module', '', ''),
				'method' => 'filterListCount',
			),
			'filterModuleCount' => array(
				'reqType' => 'GET',
				'path' => array('PX_NeedToDoViewerComments', 'filter', 'count'),
				'pathVars' => array('module', '', ''),
				'method' => 'getFilterListCount',
			),
			'filterModuleById' => array(
				'reqType' => 'GET',
				'path' => array('PX_NeedToDoViewerComments', 'filter', '?'),
				'pathVars' => array('module', '', 'record'),
				'method' => 'filterById',
			),
		);
	}

	public function filterListSetup(ServiceBase $api, array $args, $acl = 'list'){
		if (!empty($args['filter']) && is_array($args['filter'])){
		    $customFilters = $args['filter'];
            $args['filter'] = $this->removeAllCustomFilters($args['filter']);
		}

		list($args, $q, $options, $seed) = parent::filterListSetup($api, $args, $acl);


        if(!empty($customFilters)){
            $this->applyParticipantsFilter($customFilters, $q);
            $this->applyMentionedFilter($customFilters, $q);
            $this->applyNewMessagesFilter($customFilters, $q);
            $this->applyMyNewMessagesFilter($customFilters, $q);
            $this->applyMySubscriptionsFilter($customFilters, $q);
            $this->applySubscriptionFilter($customFilters, $q);
            $this->applyRelevantToFilter($customFilters, $q);
            $this->applyRelevantToMeFilter($customFilters, $q);
            $this->applyNeedToDoEventFilter($customFilters, $q);
        }

        return array($args, $q, $options, $seed);
	}

    protected function removeAllCustomFilters($filterList){
        foreach($filterList as $key => $filter){
            foreach($filter as $fieldName => $filterDefinition){
                if(!empty($fieldName) && $this->isElementDeprecated($fieldName)){
                    unset($filterList[$key][$fieldName]);
                }
            }
        }
        return $filterList;
    }

    private function isElementDeprecated($key){
        return is_string($key) && in_array($key, array('participant_id', 'mentioned_id', 'has_new_messages_id', '$my_new_messages', 'is_subscribed_to_id',
                '$my_subscriptions', 'is_relevant_to_id', '$relevant_to_me', 'filter_target_module', 'filter_object_module', 'filter_need_to_do_event_type_id'));
    }

    protected function applyParticipantsFilter(&$filterList, SugarQuery $query){
        foreach($filterList as $key => $filter){
            foreach($filter as $fieldName => $filterDefinition){
                if(!empty($fieldName) && $fieldName == 'participant_id'){
                    foreach ($filterDefinition as $operator => $ids) {
                        if (!in_array($operator, array('$in', '$not_in')) || empty($ids) || !is_array($ids))
                            continue;
                        $subQuery = new SugarQuery();
                        $subQuery->from(BeanFactory::getBean('PX_NeedToDoViewerComments'));
                        $commentUsersJoinA = $subQuery->join('dialog_users', array('joinType' => 'INNER'))->joinName();
                        $commentUsersJoinM2MA = $commentUsersJoinA . '_m2m';
                        $subQuery->where()->in($commentUsersJoinM2MA.'.user_id', $ids);
                        $subQuery->where()->equals($commentUsersJoinM2MA.'.type', 'author');
                        $subQuery->select(array('id'));

                        if($operator == '$in'){
                            $query->where()->in('id', $subQuery);
                        }else{
                            $query->where()->notIn('id', $subQuery);
                        }

                    }

                    unset($filterList[$key][$fieldName]);
                }
            }
        }
    }

    protected function applyMentionedFilter(&$filterList, SugarQuery $query){
        foreach($filterList as $key => $filter){
            foreach($filter as $fieldName => $filterDefinition){
                if(!empty($fieldName) && $fieldName == 'mentioned_id'){
                    foreach ($filterDefinition as $operator => $ids) {
                        if (!in_array($operator, array('$in', '$not_in')) || empty($ids) || !is_array($ids))
                            continue;
                        $subQuery = new SugarQuery();
                        $subQuery->from(BeanFactory::getBean('PX_NeedToDoViewerComments'));
                        $commentUsersJoinA = $subQuery->join('dialog_users', array('joinType' => 'INNER'))->joinName();
                        $commentUsersJoinM2MA = $commentUsersJoinA . '_m2m';
                        $subQuery->where()->in($commentUsersJoinM2MA.'.user_id', $ids);
                        $subQuery->where()->equals($commentUsersJoinM2MA.'.type', 'mentioned');
                        $subQuery->select(array('id'));

                        if($operator == '$in'){
                            $query->where()->in('id', $subQuery);
                        }else{
                            $query->where()->notIn('id', $subQuery);
                        }

                    }

                    unset($filterList[$key][$fieldName]);
                }
            }
        }
    }

    protected function applyNewMessagesFilter(&$filterList, SugarQuery $query){
        foreach($filterList as $key => $filter){
            foreach($filter as $fieldName => $filterDefinition){
                if(!empty($fieldName) && $fieldName == 'has_new_messages_id'){
                    foreach ($filterDefinition as $operator => $ids) {
                        if (!in_array($operator, array('$in', '$not_in')) || empty($ids) || !is_array($ids))
                            continue;
                        $subQuery = new SugarQuery();
                        $subQuery->from(BeanFactory::getBean('PX_NeedToDoViewerComments'));
                        $commentUsersJoinA = $subQuery->join('dialog_users', array('joinType' => 'INNER'))->joinName();
                        $commentUsersJoinM2MA = $commentUsersJoinA . '_m2m';
                        $subQuery->where()->in($commentUsersJoinM2MA.'.user_id', $ids);
                        $subQuery->where()->equals($commentUsersJoinM2MA.'.state', 'new');
                        $subQuery->select(array('id'));

                        if($operator == '$in'){
                            $query->where()->in('id', $subQuery);
                        }else{
                            $query->where()->notIn('id', $subQuery);
                        }

                    }

                    unset($filterList[$key][$fieldName]);
                }
            }
        }
    }

    protected function applyMyNewMessagesFilter(&$filterList, SugarQuery $query){
        foreach($filterList as $key => $filter){
            foreach($filter as $fieldName => $filterDefinition){
                if(!empty($fieldName) && $fieldName == '$my_new_messages'){
                    global $current_user;

                    $subQuery = new SugarQuery();
                    $subQuery->from(BeanFactory::getBean('PX_NeedToDoViewerComments'));
                    $commentUsersJoinA = $subQuery->join('dialog_users', array('joinType' => 'INNER'))->joinName();
                    $commentUsersJoinM2MA = $commentUsersJoinA . '_m2m';
                    $subQuery->where()->equals($commentUsersJoinM2MA.'.state', 'new');
                    $subQuery->where()->equals($commentUsersJoinM2MA.'.user_id', $current_user->id);
                    $subQuery->select(array('id'));
                    $query->where()->in('id', $subQuery);

                    unset($filterList[$key][$fieldName]);
                }
            }
        }
    }

    protected function applyMySubscriptionsFilter(&$filterList, SugarQuery $query){
        foreach($filterList as $key => $filter){
            foreach($filter as $fieldName => $filterDefinition){
                if(!empty($fieldName) && $fieldName == 'is_subscribed_to_id'){
                    foreach ($filterDefinition as $operator => $ids) {
                        if (!in_array($operator, array('$in', '$not_in')) || empty($ids) || !is_array($ids))
                            continue;

                        $subscribedJTA = 'subscribed_users_'.$key.'_'.$operator;
                        $subscribedTableSQL = "(SELECT id, created_by, parent_type, parent_id from subscriptions WHERE deleted=0)";
                        $query->join[$subscribedJTA] = $query->joinTable($subscribedTableSQL, array('alias' => $subscribedJTA, 'joinType' => 'LEFT'));
                        $query->join[$subscribedJTA]->on()->equalsField($subscribedJTA . '.parent_id', 'PX_NeedToDoViewerComments.id')
                            ->equals($subscribedJTA . '.parent_type', 'PX_NeedToDoViewerComments');
                        if($operator == '$in') {
                            $query->where()->in($subscribedJTA . '.created_by', $ids);
                        }else{
                            $query->where()->notIn($subscribedJTA . '.created_by', $ids);
                        }
                        unset($filterList[$key][$fieldName]);

                    }
                }
            }
        }
    }

    protected function applySubscriptionFilter(&$filterList, SugarQuery $query, $subscribedJTA = ''){
        foreach($filterList as $key => $filter){
            foreach($filter as $fieldName => $filterDefinition){
                if(!empty($fieldName) && $fieldName == '$my_subscriptions'){
                    global $current_user;
                    if(empty($subscribedJTA)){
                        $subscribedJTA = 'my_subscriptions_'.$key;
                        $subscribedTableSQL = "(SELECT id, created_by, parent_type, parent_id from subscriptions WHERE deleted=0)";
                        $query->join[$subscribedJTA] = $query->joinTable($subscribedTableSQL, array('alias' => $subscribedJTA, 'joinType' => 'LEFT'));
                        $query->join[$subscribedJTA]->on()->equalsField($subscribedJTA . '.parent_id', 'id')
                            ->equals($subscribedJTA . '.parent_type', 'PX_NeedToDoViewerComments');
                    }

                    $query->where()->equals($subscribedJTA.'.created_by', $current_user->id);
                    unset($filterList[$key][$fieldName]);
                }
            }
        }
    }

    protected function applyRelevantToFilter(&$filterList, SugarQuery $query){
        foreach($filterList as $key => $filter){
            foreach($filter as $fieldName => $filterDefinition){
                if(!empty($fieldName) && $fieldName == 'is_relevant_to_id'){
                    foreach ($filterDefinition as $operator => $ids) {
                        if (!in_array($operator, array('$in', '$not_in')) || empty($ids) || !is_array($ids))
                            continue;

                        $aliasKey = 'is_relevant_to_id_' . $key . '_' . substr($operator, 1);
//                      $this->relevantFilterQuery($query, $operator, $ids, $aliasKey);
                        $subQuery = new SugarQuery();
                        $subQuery->from(BeanFactory::getBean('PX_NeedToDoViewerComments'));
                        $commentUsersJoinA = $subQuery->join('dialog_users', array('joinType' => 'LEFT'))->joinName();
//                      $SubscribedUsersJoinA = $subQuery->join('following_link', array('joinType' => 'LEFT'))->joinName();

                        $subscribedJTA = 'subscribed_' . $aliasKey;
                        $subscribedTableSQL = "(SELECT id, created_by, parent_type, parent_id from subscriptions WHERE deleted=0)";
                        $subQuery->join[$subscribedJTA] = $subQuery->joinTable($subscribedTableSQL, array('alias' => $subscribedJTA, 'joinType' => 'LEFT'));
                        $subQuery->join[$subscribedJTA]->on()->equalsField($subscribedJTA . '.parent_id', 'PX_NeedToDoViewerComments.id')
                            ->equals($subscribedJTA . '.parent_type', 'PX_NeedToDoViewerComments');
//                            ->in($subscribedJTA . '.created_by', $ids);

                        $subQuery->select(array('id'));

                        $orWhere = $subQuery->where()->queryOr();
                        $orWhere->in($commentUsersJoinA.'.id', $ids);
//                        $orWhere->isNotEmpty($subscribedJTA . '.id');
                        $orWhere->in($subscribedJTA . '.created_by', $ids);

                        if($operator == '$in') {
                            $query->where()->in('id', $subQuery);
                        }else{
                            $query->where()->notIn('id', $subQuery);
                        }

                        unset($filterList[$key][$fieldName]);

                    }
                }
            }
        }
    }

    protected function applyRelevantToMeFilter(&$filterList, SugarQuery $query){
        foreach($filterList as $key => $filter){
            foreach($filter as $fieldName => $filterDefinition){
                if(!empty($fieldName) && $fieldName == '$relevant_to_me'){
                    global $current_user;
                    $operator = '$in';
                    $ids = array($current_user->id);
//                    $this->relevantFilterQuery($query, $operator, $ids, $aliasKey);

                    $subQuery = new SugarQuery();
                    $subQuery->from(BeanFactory::getBean('PX_NeedToDoViewerComments'));
                    $commentUsersJoinA = $subQuery->join('dialog_users', array('joinType' => 'INNER'))->joinName();
                    $subQuery->select(array('id'));
                    $subQuery->where()->in($commentUsersJoinA.'.id', $ids);


                    $orWhere = $query->where()->queryOr();
                    $orWhere->in('id', $subQuery);
                    $orWhere->equals('subscribed_users.created_by', $current_user->id);

                    unset($filterList[$key][$fieldName]);

                }
            }
        }
    }

    protected function applyNeedToDoEventFilter(&$filterList, SugarQuery $query, $needToDoEventJoinAlias = ''){
        foreach($filterList as $key => $filter){
            foreach($filter as $fieldName => $filterDefinition){
                if(!empty($fieldName) && in_array($fieldName,  array('filter_target_module', 'filter_object_module', 'filter_need_to_do_event_type_id'))){
                    foreach ($filterDefinition as $operator => $values) {
                        if (in_array($operator, array('$in', '$not_in')) && (empty($values) || !is_array($values)))
                            continue;

                        if(empty($needToDoEventJoinAlias)){
                            $needToDoEventJoin = $query->join('need_to_do_event', array(/*'joinType' => 'INNER', 'alias' => $joinName*/));
                            $needToDoEventJoinAlias = $needToDoEventJoin->joinName();
                        }

                        $fieldNameForQuery = $needToDoEventJoinAlias . '.';
                        if($fieldName == 'filter_target_module'){
                            $fieldNameForQuery .= 'parent_type';
                        }elseif($fieldName == 'filter_object_module'){
                            $fieldNameForQuery .= 'object_type';
                        }else{
                            $fieldNameForQuery .= 'event_id';
                        }

                        if($operator == '$in') {
                            $query->where()->in($fieldNameForQuery, $values);
                        }elseif($operator == '$in'){
                            $query->where()->notIn($fieldNameForQuery, $values);
                        }elseif($operator == '$empty'){
                            $query->where()->isEmpty($fieldNameForQuery);
                        }elseif($operator == '$not_empty'){
                            $query->where()->isNotEmpty($fieldNameForQuery);
                        }

                        unset($filterList[$key][$fieldName]);
                    }
                }
            }
        }
    }

    protected function formatBean(ServiceBase $api, array $args, SugarBean $bean, array $options = array())
    {
        $data = parent::formatBean($api, $args, $bean, $options);
        if(!empty($data['id']) && !empty($data['parent_type'])){
            if($data['parent_type'] == 'PX_NeedToDoEvents'){
                global $current_user;
                $countQuery = new SugarQuery();
                $countQuery->from(BeanFactory::getBean('PX_NeedToDoViewerComments'));

                $dialogUsers = $countQuery->join('dialog_users', array('joinType' => 'INNER'))->joinName();
                $dialogUsersM2M = $dialogUsers . '_m2m';
                $countQuery->where()->equals($dialogUsersM2M . '.state', 'new');
                $countQuery->where()->equals($dialogUsers.'.id', $current_user->id);
                $countQuery->where()->equals('px_needtodoviewercomments.id', $data['id']);
                $countQuery->select()->selectReset()->addField($dialogUsersM2M.'.comment_id', array('alias' => 'unread_comment_id'));
                $countQuery->limit = null;

                $countQueryResult = $countQuery->execute();

                $queryResult = $countQueryResult;
                $data['count_new_msg'] = count($queryResult);
                $data['has_new_messages'] = $data['count_new_msg'] > 0 ? true : false;

                if($data['has_new_messages']){

                    $data['unread_comment_ids'] = array();
                    foreach($queryResult as $item){
                        $data['unread_comment_ids'][] = $item['unread_comment_id'];
                    }
                }
            }
        }

        return $data;
    }

    public function checkMaxListLimit($limit)
    {
        return -1;
    }

}